/*
 * BinarySlamMap.cpp
 *
 *  Created on: Jul 31, 2009
 *      Author: jwienke
 */

#include "BinarySlamMap.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

std::string BinarySlamMap::getBaseTag() {
	return "BINARYSLAMMAP";
}

std::string BinarySlamMap::getTemplate() {

	return "<" + getBaseTag() + ">"
		+ "<GENERATOR/>"
		+ airobots::Timestamp::getTemplate()
		+ "<POSITION ref=\"world\" kind=\"absolute\" x=\"\" y=\"\" theta=\"\" />"
		+ "<MAPPROPERTIES width=\"\" height=\"\" xorigin=\"\" yorigin=\"\" resolution=\"\"/>"
		+ "</" + getBaseTag() + ">";

}

BinarySlamMap::BinarySlamMap() :
	PositionData(0, 0, 0), dynamicGridMap("unknown") {

}

BinarySlamMap::BinarySlamMap(string generator) :
	PositionData(0, 0, 0), dynamicGridMap(generator) {

}

BinarySlamMap::BinarySlamMap(const BinarySlamMap &map) :
	PositionData(map), dynamicGridMap(map.dynamicGridMap) {

}

BinarySlamMap& BinarySlamMap::operator=(BinarySlamMap const& map) {
	if (&map != this) {
		setGenerator(map.getGenerator());
		setTimestamp(map.getTimestamp());
		setUri(map.getUri());
		setSizeX(map.getSizeX());
		setSizeY(map.getSizeY());
		setOriginX(map.getOriginX());
		setOriginY(map.getOriginY());
		setResolution(map.getResolution());
		setGridMap((GridMap&)(map.getGridMap()));
		setX(map.getX());
		setY(map.getY());
	}
	return *this;
}

BinarySlamMap::~BinarySlamMap() {
}

std::string BinarySlamMap::getUri() const{
	return dynamicGridMap.getUri();
}

void BinarySlamMap::setUri(std::string uri){
	dynamicGridMap.setUri(uri);
}

unsigned int BinarySlamMap::getSizeX() const{
	return dynamicGridMap.getSizeX();
}

void BinarySlamMap::setSizeX(unsigned int sizeX){
	dynamicGridMap.setSizeX(sizeX);
}

unsigned int BinarySlamMap::getSizeY() const{
	return dynamicGridMap.getSizeY();
}

void BinarySlamMap::setSizeY(unsigned int sizeY){
	dynamicGridMap.setSizeY(sizeY);
}

unsigned int BinarySlamMap::getOriginX() const{
	return dynamicGridMap.getOriginX();
}

void BinarySlamMap::setOriginX(unsigned int originX){
	dynamicGridMap.setOriginX(originX);
}

unsigned int BinarySlamMap::getOriginY() const{
	return dynamicGridMap.getOriginY();
}

void BinarySlamMap::setOriginY(unsigned int originY){
	dynamicGridMap.setOriginY(originY);
}

float BinarySlamMap::getResolution() const{
	return dynamicGridMap.getResolution();
}

void BinarySlamMap::setResolution(float resolution){
	dynamicGridMap.setResolution(resolution);
}

const GridMap& BinarySlamMap::getGridMap() const{
	return dynamicGridMap.getGridMap();
}

void BinarySlamMap::setGridMap(GridMap &gridMap){
	dynamicGridMap.setGridMap(gridMap);
}

void BinarySlamMap::setGridMap(const std::vector<unsigned char> &charArray,
		unsigned int width, unsigned int height){
	dynamicGridMap.setGridMap(charArray, width, height);
}

void BinarySlamMap::setGridMap(const std::vector<float> &floatArray, unsigned int width,
		unsigned int height){
	dynamicGridMap.setGridMap(floatArray, width, height);
}

void BinarySlamMap::sanitizeLocation(xmltio::Location &loc) const {
	dynamicGridMap.sanitizeLocation(loc);
}

void BinarySlamMap::fillInto(xmltio::Location &loc) const {

	PositionData::fillInto(loc);

	loc[XPath("MAPPROPERTIES/@width")] = getSizeX();
	loc[XPath("MAPPROPERTIES/@height")] = getSizeY();
	loc[XPath("MAPPROPERTIES/@xorigin")] = getOriginX();
	loc[XPath("MAPPROPERTIES/@yorigin")] = getOriginY();
	loc[XPath("MAPPROPERTIES/@resolution")] = getResolution();

	loc["uri"] = getUri();

}

void BinarySlamMap::fromLocation(const xmltio::Location &loc_, BinarySlamMap &map) {

	PositionData::fromLocation(loc_, map);

	map.setUri(extract<string> (loc_["uri"]));

	map.setSizeX(extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@width")]));
	map.setSizeY(extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@height")]));

	map.setOriginX(
			extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@xorigin")]));
	map.setOriginY(
			extract<unsigned int> (loc_[XPath("MAPPROPERTIES/@yorigin")]));

	map.setResolution(extract<float> (loc_[XPath("MAPPROPERTIES/@resolution")]));

}

