/*
 * Face.h
 *
 *  Created on: Aug 18, 2009
 *      Author: jwienke
 */

#ifndef FACE_H_
#define FACE_H_

#include "Type.h"
#include "Percept.h"

namespace airobots {

/**
 * Face percept representation.
 *
 * @author jwienke
 */
class Face: public Percept {

public:

	static std::string getBaseTag();
	static std::string getTemplate();
	static std::string getPerceptName();

	Face();
	virtual ~Face();

	// TODO units and references?

	float getAngle() const;
	void setAngle(float angle);

	float getHeight() const;
	void setHeight(float height);

	float getDistance() const;
	void setDistance(float distance);

	float getAngleVariance() const;
	void setAngleVariance(float angleVariance);

	float getHeightVariance() const;
	void setHeightVariance(float heightVariance);

	float getDistanceVariance() const;
	void setDistanceVariance(float distanceVariance);

	float getReliability() const;
	void setReliability(float reliability);

	float getFaceWidthAngle() const;
	void setFaceWidthAngle(float faceWidthAngle);

	int getLineOfSight() const;
	void setLineOfSight(int lineOfSight);

	/**
	 * Returns the face x position in pixels (top left corner).
	 */
	unsigned int getImagePositionX() const;
	void setImagePositionX(unsigned int x);

	/**
	 * Returns the face y position in pixels (top left corner).
	 */
	unsigned int getImagePositionY() const;
	void setImagePositionY(unsigned int y);

	/**
	 * Returns the width of the face percept in the image in pixels.
	 */
	unsigned int getImageWidth() const;
	void setImageWidth(unsigned int imageWidth);

	/**
	 * Returns the height of the face percept in the image in pixels.
	 */
	unsigned int getImageHeight() const;
	void setImageHeight(unsigned int imageHeight);

	bool operator==(const Face &other) const;

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Face &type);

private:

	float angle;
	float angleVariance;
	float height;
	float heightVariance;
	float distance;
	float distanceVariance;
	float reliability;
	float faceWidthAngle;
	int lineOfSight;
	unsigned int imagePositionX;
	unsigned int imagePositionY;
	unsigned int imageWidth;
	unsigned int imageHeight;

};

std::ostream& operator<<(std::ostream& o, Face const& face);

}

#endif /* FACE_H_ */
