/*
 * FaceIdentificationList.h
 *
 *  Created on: Nov 12, 2010
 *      Author: ltwardon
 */

#ifndef FACEIDENTIFICATIONLIST_H_
#define FACEIDENTIFICATIONLIST_H_

#include "Type.h"
#include <list>

namespace airobots
{

/**
 * List of all face identifications found.
 */
class FaceIdentificationList: public Type
{
public:

	/**
	 * One single result of the face identification.
	 */
	class FaceIdentification
	{
	private:
		int regionX;
		int regionY;
		int regionWidth;
		int regionHeight;
		int classId;

	public:
		FaceIdentification();

	    int getClassId() const;
	    int getRegionHeight() const;
	    int getRegionWidth() const;
	    int getRegionX() const;
	    int getRegionY() const;
	    void setClassId(int classId);
	    void setRegionHeight(int regionHeight);
	    void setRegionWidth(int regionWidth);
	    void setRegionX(int regionX);
	    void setRegionY(int regionY);
	    bool isIdentified() const;
	};

	FaceIdentificationList();

	static std::string getBaseTag();

	static std::string getTemplate();

	int size();

	std::list<FaceIdentification> getIdentifications();

	void sanitizeLocation(xmltio::Location &loc) const;
	virtual void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, FaceIdentificationList &type);

private:
	std::list<FaceIdentification> identifications;

	void addIdentification(FaceIdentification identification);
};

}

#endif /* FACEIDENTIFICATIONLIST_H_ */
