#include <exception>
#include <stdexcept>
#include <string>

#include "GlobalPlannerStateChange.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

std::string GlobalPlannerStateChange::getBaseTag() {
	return "GLOBALPLANNERSTATECHANGE";
}

std::string GlobalPlannerStateChange::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<STATE/></" + getBaseTag() + ">";
}

GlobalPlannerStateChange::GlobalPlannerStateChange() {
}

GlobalPlannerStateChange::GlobalPlannerStateChange(std::string generator,
		GlobalPlannerStateChange::State newState) :
	Type(generator), newState(newState) {

}

GlobalPlannerStateChange::State GlobalPlannerStateChange::getNewState() const {
	return newState;
}

void GlobalPlannerStateChange::setNewState(
		GlobalPlannerStateChange::State newState) {
	this->newState = newState;
}

void GlobalPlannerStateChange::sanitizeLocation(xmltio::Location &loc) const {

}

void GlobalPlannerStateChange::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	// Convert the state to a string
	string stateString;
	switch (getNewState()) {
	case GlobalPlannerStateChange::PLANNING:
		stateString = "PLANNING";
		break;
	case GlobalPlannerStateChange::FOLLOWING_PLAN:
		stateString = "FOLLOWING_PLAN";
		break;
	case GlobalPlannerStateChange::PATH_BLOCKED:
		stateString = "PATH_BLOCKED";
		break;
	default:
		runtime_error e("Illegal enum value. Fix convert(PlannerState)?");
		throw e;
		break;
	}

	// Set the value of the <PlannerStateChange> element to that string
	loc[XPath("STATE")] = stateString;

}

void GlobalPlannerStateChange::fromLocation(const xmltio::Location &loc_, GlobalPlannerStateChange &change) {

	Type::fromLocation(loc_, change);

	string stateString = extract<string> (loc_[XPath(
			"STATE")]);
	GlobalPlannerStateChange::State state;
	if (stateString == "PLANNING") {
		state = GlobalPlannerStateChange::PLANNING;
	} else if (stateString == "FOLLOWING_PLAN") {
		state = GlobalPlannerStateChange::FOLLOWING_PLAN;
	} else if (stateString == "PATH_BLOCKED") {
		state = GlobalPlannerStateChange::PATH_BLOCKED;
	} else {
		throw ParsingException("Illegal enum value for State",
				loc_.getDocumentText(true));
	}
	change.setNewState(state);

}
