/*
 * plannerstate.h
 *
 *  Created on: Jun 25, 2009
 *      Author: jwienke
 */

#ifndef GLOBALPLANNERSTATECHANGE_H_
#define GLOBALPLANNERSTATECHANGE_H_

#include "Type.h"
#include "Timestamp.h"

namespace airobots {

/**
 * A state change of the global planner.
 */
class GlobalPlannerStateChange: public Type {
public:

	static std::string getBaseTag();

	static std::string getTemplate();

	/**
	 * Possible states of a global planner.
	 */
	enum State {

		/**
		 * The planner now begins planning a new path.
		 */
		PLANNING,

		/**
		 * Planning finished successfully and the robots is now trying to move
		 * along the planned path. In this state the planner can switch back
		 * to <code>PLANNING</code> every time it detects a new obstacle on the
		 * planned path.
		 */
		FOLLOWING_PLAN,

		/**
		 * The planner could not find a path to the desired location. This state
		 * is only left if a new goal is specified.
		 */
		PATH_BLOCKED

	};

	GlobalPlannerStateChange();
	GlobalPlannerStateChange(std::string generator, State newState);

	State getNewState() const;
	void setNewState(State newState);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, GlobalPlannerStateChange &type);

private:
	State newState;

};

}

#endif
