/*
 * MapRegion.h
 *
 *  Created on: Oct 13, 2010
 *      Author: ltwardon
 */

#ifndef MAPREGION_H_
#define MAPREGION_H_

#include <string>
#include <list>
#include "MapPoint.h"
#include "PositionData.h"
#include "NavigationGoalData.h"

namespace airobots {

class MapRegion {

public:

	static const std::string POLYGONE_TYPE;
	static const std::string Rectangle_TYPE;

	MapRegion();
	MapRegion(std::string type);
	MapRegion(std::string type, std::string label);
	MapRegion(std::string type, std::string label, std::string group);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points, NavigationGoalData viewPoint);
	MapRegion(std::string type, std::string label, std::string group,
			std::list<MapPoint> points, NavigationGoalData viewPoint, double height);

	void addPoint(double x, double y);
	void addPoint(MapPoint point);
	std::string getType() const;
	void setType(std::string type);
	std::string getLabel() const;
	void setLabel(std::string label);
	std::string getGroup() const;
	void setGroup(std::string group);
	std::list<MapPoint> getPoints() const;
	void setPoints(std::list<MapPoint> points);
	NavigationGoalData getViewPoint() const;
	void setViewPoint(NavigationGoalData viewPoint);
	double getHeight() const ;
	void setHeight(double height);
	void removePoint(double x, double y);
	int getPointCount() const;
	PositionData getPositionData() const;
	bool isViewPointSet() const;
	bool contains(double x, double y) const;

private:

	std::string type;
	std::string label;
	std::string group;
	std::list<MapPoint> points;
	NavigationGoalData viewPoint;
	double height;
	bool viewPointSet;

};

}

#endif /* MAPREGION_H_ */

