/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     navgoaldata.h
 * @author   Roland Philippsen
 * @date     14.09.2006
 * @brief    Converting and extracting navgoal data to and from xml-structure
 */

#ifndef NAVGOALDATA_H
#define NAVGOALDATA_H

#include <string>
#include <cmath>

#include <xmltio/xmltio.hpp>

#include "PositionData.h"

namespace airobots {

class NavigationGoalData: public PositionData {

public:

	enum NavgoalType {
		LOCAL, GLOBAL
	};

	static const float DEFAULT_DR = 0.3;
	static const float DEFAULT_DTHETA = M_PI;

	static std::string getBaseTag();

	static std::string getTemplate();

	NavigationGoalData();
	NavigationGoalData(std::string generator, float x, float y, float theta,
			float dr = DEFAULT_DR, float dtheta = DEFAULT_DTHETA,
			NavgoalType type = LOCAL);

	float getDR() const;
	float getDTheta() const;
	NavgoalType getType() const;
	void setDR(float dR);
	void setDTheta(float dTheta);
	void setType(NavgoalType type);

	bool operator==(const NavigationGoalData &other) const;

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, NavigationGoalData &type);

private:
	float dR;
	float dTheta;
	NavgoalType type;

};

std::ostream& operator<<(std::ostream& o, NavigationGoalData const& ngd);

}

#endif // NAVGOALDATA_H
