#ifndef OBJECTDATA_H_
#define OBJECTDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "Type.h"

namespace airobots {

/**
 * Results of the object recognition. This is a very general base class.
 *
 * @author lziegler
 */
class ObjectData: public Type {

public:

	class Hypothesis: public Type {
	private:
		/**
		 * Reliability of the recognition result.
		 */
		double reliability;

	public:

		static std::string getBaseTag();

		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);
		Hypothesis(Hypothesis &h);

		double getReliability() const;
		void setReliability(const double &reliability);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	ObjectData();
	ObjectData(std::string generator);
	~ObjectData();

	std::string getClassLabel() const;
	void setClassLabel(const std::string &label);

	void addHypothesis(Hypothesis* h);
	void clearHypotheses();
	std::vector<Hypothesis*> getHypotheses() const;

	virtual bool operator==(const ObjectData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, ObjectData &type);

private:

	/**
	 * Name of this object.
	 */
	std::string classLabel;

	std::vector<Hypothesis*> hypotheses;

};

}

#endif /* OBJECTDATA_H_ */
