#include "ObjectLocationData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string ObjectLocationData::getBaseTag() {
	return "OBJECTLOCATION";
}

std::string ObjectLocationData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectLocationData::getTemplate() {
	return "<" + getBaseTag() + "><CLASS/></" + getBaseTag() + ">";
}

std::string ObjectLocationData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING><SENSORTYPE/>"
			+ "<POLYGON/></" + getBaseTag() + ">";
}

ObjectLocationData::ObjectLocationData() :
	ObjectData() {
}

ObjectLocationData::Hypothesis::Hypothesis() :
	ObjectData::Hypothesis() {
}

ObjectLocationData::ObjectLocationData(std::string generator) :
	ObjectData(generator) {
}

ObjectLocationData::Hypothesis::Hypothesis(std::string generator) :
	ObjectData::Hypothesis(generator) {
}

void ObjectLocationData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectLocationData::Hypothesis::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT");
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectLocationData::fillInto(xmltio::Location &loc) const {

	ObjectData::fillInto(loc);
	ObjectData::sanitizeLocation(loc);

	vector<ObjectData::Hypothesis*> h = getHypotheses();
	for (vector<ObjectData::Hypothesis*>::const_iterator it = h.begin(); it
			!= h.end(); it++) {

		try {
			Hypothesis* h0 = dynamic_cast<Hypothesis*> (*it);

			Location hyp(Hypothesis::getTemplate(), XPath(
					Hypothesis::getBaseTag()));

			h0->fillInto(hyp);
			loc.add(hyp);
		} catch (bad_cast e) {
			continue;
		}
	}
}

void ObjectLocationData::Hypothesis::fillInto(xmltio::Location &loc) const {

	ObjectData::Hypothesis::fillInto(loc);
	loc[XPath("SENSORTYPE")] = getSensorType();

	Location polyLoc = Location(loc, "POLYGON");

	for (vector<pair<int, int> >::const_iterator it = polygon.begin(); it
			!= polygon.end(); it++) {
		Location newPoint("<POINT x=\"\" y=\"\" />", "POINT");
		newPoint[XPath("@x")] = it->first;
		newPoint[XPath("@y")] = it->second;
		polyLoc.add(newPoint);
	}
}

void ObjectLocationData::fromLocation(const xmltio::Location &loc_,
		ObjectLocationData &object) {

	ObjectData::fromLocation(loc_, object);

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_); iter; ++iter) {
		Hypothesis* hyp = new Hypothesis();
		Hypothesis::fromLocation(*iter, *hyp);
		object.clearHypotheses();
		object.addHypothesis(hyp);
	}
}

void ObjectLocationData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	ObjectData::Hypothesis::fromLocation(loc_, object);

	object.setSensorType(extract<string> (loc_[XPath("SENSORTYPE")]));

	vector<pair<int, int> > poly;
	for (XPathIterator iter = XPath("POLYGON/POINT").evaluate(loc_); iter; ++iter) {

		int x = extract<double> ((*iter)[XPath("@x")]);
		int y = extract<double> ((*iter)[XPath("@y")]);
		poly.push_back(pair<int, int> (x, y));
	}

	object.setPolygon(poly);
}

string ObjectLocationData::Hypothesis::getSensorType() const {
	return sensorType;
}
void ObjectLocationData::Hypothesis::setSensorType(const string &sensorType) {
	ObjectLocationData::Hypothesis::sensorType = sensorType;
}

vector<pair<int, int> > ObjectLocationData::Hypothesis::getPolygon() const {
	return polygon;
}
void ObjectLocationData::Hypothesis::setPolygon(
		const vector<pair<int, int> > &polygon) {
	ObjectLocationData::Hypothesis::polygon = polygon;
}

bool ObjectLocationData::operator==(const ObjectLocationData &other) const {

	bool isHyp = true;

	try {
		vector<Hypothesis*> h0 = getLocationHypotheses();
		vector<Hypothesis*> h1 = other.getLocationHypotheses();

		if (h0.size() != h1.size()) {
			return false;
		}

		vector<Hypothesis*>::iterator it0 = h0.begin();
		for (vector<Hypothesis*>::iterator it1 = h1.begin(); it1
				!= h1.end(); it1++) {

			Hypothesis hyp0 = **it0;
			Hypothesis hyp1 = **it1;

			if (!(hyp0 == hyp1)) {
				isHyp = false;
			}
			it0++;
		}

		return ObjectData::operator==(other) && isHyp;
	} catch (bad_cast e) {
		return false;
	}

}

bool ObjectLocationData::Hypothesis::operator==(const Hypothesis &other) const {

	bool isPoly = true;
	if (other.getPolygon().size() != getPolygon().size()) {
		isPoly = false;
	} else {
		for (int i = 0; i < (int) getPolygon().size(); i++) {
			if (getPolygon()[i].first != other.getPolygon()[i].first
					|| getPolygon()[i].second != other.getPolygon()[i].second) {
				isPoly = false;
			}
		}
	}
	return ObjectData::Hypothesis::operator==(other) && other.getSensorType()
			== getSensorType() && isPoly;

}

vector<ObjectLocationData::Hypothesis*> ObjectLocationData::getLocationHypotheses() const {

	vector<ObjectData::Hypothesis*> h0 = ObjectData::getHypotheses();
	vector<Hypothesis*> h1;

	for (vector<ObjectData::Hypothesis*>::iterator it = h0.begin(); it
			!= h0.end(); it++) {
		try {
			Hypothesis* hyp = dynamic_cast<Hypothesis*> (*it);
			h1.push_back(hyp);
		} catch (bad_cast e) {
			continue;
		}
	}
	return h1;
}
