#ifndef OBJECTLOCATIONDATA_H_
#define OBJECTLOCATIONDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "ObjectData.h"

namespace airobots {

/**
 * Results of the object recognition.
 *
 * @author lziegler
 */
class ObjectLocationData: public ObjectData {

public:

	class Hypothesis: public ObjectData::Hypothesis {
	private:
		std::string sensorType;
		std::vector<std::pair<int, int> > polygon;

	public:

		static std::string getBaseTag();

		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);

		std::string getSensorType() const;
		void setSensorType(const std::string &label);

		std::vector<std::pair<int, int> > getPolygon() const;
		void setPolygon(const std::vector<std::pair<int, int> > &polygon);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	ObjectLocationData();
	ObjectLocationData(std::string generator);

	std::vector<Hypothesis*> getLocationHypotheses() const;

	virtual bool operator==(const ObjectLocationData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			ObjectLocationData &type);

};

}

#endif /* OBJECTLOCATIONDATA_H_ */
