#include "Path3DData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string Path3DData::getBaseTag() {
	return "PATH3D";
}

std::string Path3DData::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<SCOPE/><PRECISEPOLYGON3D /></" + getBaseTag() + ">";
}

Path3DData::Path3DData() :
	Type() {
}

Path3DData::Path3DData(std::string generator) :
	Type(generator) {
}

void Path3DData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT3D");
	while (XPathIterator i = xp.evaluate(loc[XPath("PRECISEPOLYGON3D")])) {
		Location l(*i);
		loc.remove(l);
	}
}

void Path3DData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);
	Location polyLoc = Location(loc, "PRECISEPOLYGON3D");
	if (scope == LOCAL) {
		loc[XPath("SCOPE")] = "LOCAL";
	} else {
		loc[XPath("SCOPE")] = "GLOBAL";
	}
	for (vector<triple<float, float, float> >::const_iterator it = polygon.begin(); it
			!= polygon.end(); it++) {
		Location newPoint("<POINT3D x=\"\" y=\"\" z=\"\" />", "POINT3D");
		newPoint[XPath("@x")] = it->first;
		newPoint[XPath("@y")] = it->second;
		newPoint[XPath("@z")] = it->third;
		polyLoc.add(newPoint);
	}
}

void Path3DData::fromLocation(const xmltio::Location &loc_, Path3DData &object) {

	Type::fromLocation(loc_, object);

	string s = extract<string> (loc_[XPath("SCOPE")]);
	if (s == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}

	vector<triple<float, float, float> > poly;
	for (XPathIterator iter = XPath("PRECISEPOLYGON3D/POINT3D").evaluate(loc_); iter; ++iter) {

		float x = extract<float> ((*iter)[XPath("@x")]);
		float y = extract<float> ((*iter)[XPath("@y")]);
		float z = extract<float> ((*iter)[XPath("@z")]);
		poly.push_back(triple<float, float, float> (x, y, z));
	}

	object.setPolygon(poly, MILLIMETER);
}

vector<triple<float, float, float> > Path3DData::scalePolygon(
		vector<triple<float, float, float> > poly, double factor) const {
	for (vector<triple<float, float, float> >::iterator it = poly.begin(); it != poly.end(); ++it) {
		it->first *= factor;
		it->second *= factor;
		it->third *= factor;
	}
	return poly;
}

vector<triple<float, float, float> > Path3DData::getPolygon(Path3DData::Unit unit) const {
	switch (unit) {
	case METER:
		return scalePolygon(polygon, 0.001);
	case CENTIMETER:
		return scalePolygon(polygon, 0.1);
	case MILLIMETER:
	default:
		return polygon;
	}
}
void Path3DData::setPolygon(const vector<triple<float, float, float> > &polygon, Unit unit) {
	switch (unit) {
	case METER:
		this->polygon = scalePolygon(polygon, 1000);
		break;
	case CENTIMETER:
		this->polygon = scalePolygon(polygon, 10);
		break;
	case MILLIMETER:
	default:
		this->polygon = polygon;
		break;
	}
}

Path3DData::Scope Path3DData::getScope() const {
	return scope;
}

void Path3DData::setScope(const Scope &scope) {
	this->scope = scope;
}

bool Path3DData::operator==(const Path3DData &other) const {

	bool isPoly = scope == other.getScope();

	try {
		vector<triple<float, float, float> > h0 = getPolygon(MILLIMETER);
		vector<triple<float, float, float> > h1 = other.getPolygon(MILLIMETER);

		if (h0.size() != h1.size()) {
			return false;
		}

		vector<triple<float, float, float> >::iterator it0 = h0.begin();
		for (vector<triple<float, float, float> >::iterator it1 = h1.begin(); it1 != h1.end(); it1++) {

			if (it0->first != it1->first || it0->second != it1->second || it0->third != it1->third) {
				isPoly = false;
			}
			it0++;
		}

		return isPoly;
	} catch (bad_cast &e) {
		return false;
	}
}
