#include "PlaneData.h"
#include <math.h>

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string PlaneData::getBaseTag() {
	return "PLANE";
}

std::string PlaneData::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<SCOPE/><TRANSLATION x=\"\" y=\"\" z=\"\" />"
			+ "<NORMALROTATION x=\"\" y=\"\" z=\"\" />" + PrecisePolygon::getTemplate() + "</" + getBaseTag() + ">";
}

PlaneData::PlaneData() :
	RegionData() {
}

PlaneData::PlaneData(std::string generator) :
	RegionData(generator) {
}

void PlaneData::sanitizeLocation(xmltio::Location &loc) const {
	RegionData::sanitizeLocation(loc);
}

void PlaneData::fillInto(xmltio::Location &loc) const {

	RegionData::fillInto(loc);

	loc[XPath("TRANSLATION/@x")] = translationX;
	loc[XPath("TRANSLATION/@y")] = translationY;
	loc[XPath("TRANSLATION/@z")] = translationZ;

	loc[XPath("NORMALROTATION/@x")] = normalRotationX;
	loc[XPath("NORMALROTATION/@y")] = normalRotationY;
	loc[XPath("NORMALROTATION/@z")] = normalRotationZ;
}

void PlaneData::fromLocation(const xmltio::Location &loc_, PlaneData &object) {

	RegionData::fromLocation(loc_, object);

	double tx = extract<double> (loc_[XPath("TRANSLATION/@x")]);
	double ty = extract<double> (loc_[XPath("TRANSLATION/@y")]);
	double tz = extract<double> (loc_[XPath("TRANSLATION/@z")]);

	object.setTranslation(tx, ty, tz, MILLIMETER);

	double rx = extract<double> (loc_[XPath("NORMALROTATION/@x")]);
	double ry = extract<double> (loc_[XPath("NORMALROTATION/@y")]);
	double rz = extract<double> (loc_[XPath("NORMALROTATION/@z")]);

	object.setNormalRotation(rx, ry, rz, RADIANT);
}

bool PlaneData::operator==(const PlaneData &other) const {

	bool region = RegionData::operator==(other);

	if (translationX != other.translationX || translationY != other.translationY || translationZ
			!= other.translationZ) {
		return false;
	}

	if (normalRotationX != other.normalRotationX || normalRotationY != other.normalRotationY
			|| normalRotationZ != other.normalRotationZ) {
		return false;
	}

	return region;
}

void PlaneData::setTranslation(double tx, double ty, double tz, LengthUnit unit) {
	translationX = UnitConverter::convert(tx, unit, MILLIMETER);
	translationY = UnitConverter::convert(ty, unit, MILLIMETER);
	translationZ = UnitConverter::convert(tz, unit, MILLIMETER);
}

double PlaneData::getTranslationX(LengthUnit unit) {
	return UnitConverter::convert(translationX, MILLIMETER, unit);
}

double PlaneData::getTranslationY(LengthUnit unit) {
	return UnitConverter::convert(translationY, MILLIMETER, unit);
}
double PlaneData::getTranslationZ(LengthUnit unit) {
	return UnitConverter::convert(translationZ, MILLIMETER, unit);
}

void PlaneData::setNormalRotation(double rx, double ry, double rz, AngleUnit unit) {
	normalRotationX = UnitConverter::convert(rx, unit, RADIANT);
	normalRotationY = UnitConverter::convert(ry, unit, RADIANT);
	normalRotationZ = UnitConverter::convert(rz, unit, RADIANT);
}
double PlaneData::getNormalRotationX(AngleUnit unit) {
	return UnitConverter::convert(normalRotationX, RADIANT, unit);
}
double PlaneData::getNormalRotationY(AngleUnit unit) {
	return UnitConverter::convert(normalRotationY, RADIANT, unit);
}
double PlaneData::getNormalRotationZ(AngleUnit unit) {
	return UnitConverter::convert(normalRotationZ, RADIANT, unit);
}
