#ifndef PlaneDATA_H_
#define PlaneDATA_H_

#include <string>

#include <xmltio/xmltio.hpp>

#include "RegionData.h"
#include "UnitConverter.h"

namespace airobots {

/**
 * This class describes an arbitrary shaped plane in a 3d space. The plane's shape is described by a
 * polygon in the x-y-layer plus a translation and rotation rule. To obtain the real 3d coordinates
 * of the given points, you will have to add a zero in the third dimension (as the polygon describes
 * the shape in the x-y-layer), rotate the points with the normal rotation angles and translate the
 * resulting points with the given translation values.
 *
 * @author lziegler
 */
class PlaneData: public RegionData {

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	virtual bool operator==(const PlaneData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PlaneData &type);

	PlaneData();
	PlaneData(std::string generator);

	void setTranslation(double tx, double ty, double tz, LengthUnit unit);
	double getTranslationX(LengthUnit unit);
	double getTranslationY(LengthUnit unit);
	double getTranslationZ(LengthUnit unit);

	void setNormalRotation(double rx, double ry, double rz, AngleUnit unit);
	double getNormalRotationX(AngleUnit unit);
	double getNormalRotationY(AngleUnit unit);
	double getNormalRotationZ(AngleUnit unit);

private:
	double translationX;
	double translationY;
	double translationZ;
	double normalRotationX;
	double normalRotationY;
	double normalRotationZ;
};

}

#endif /* PlaneDATA_H_ */
