#include "Polygon.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

Polygon::Polygon() {
	points = vector<Point2D>();
}

Polygon::~Polygon() {
}

void Polygon::fillInto(xmltio::Location & loc) const {

	Type::fillInto(loc);

	int n = points.size();
	for (int i = 0; i < n; i++) {
		Location item("<" + getItemTag() + " x=\"\" y=\"\"/>", getItemTag());
		item[XPath("@x")] = points[i].x;
		item[XPath("@y")] = points[i].y;
		loc.add(item);
	}

}



std::string Polygon::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
		+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}



void Polygon::fromLocation(const xmltio::Location & loc, Polygon & pol) {
	Type::fromLocation(loc, pol);

	for (XPathIterator i = XPath(getItemTag()).evaluate(loc); i; ++i) {

		Location pointLocation(*i);

		/*
		Location pointXLocation(pointLocation, "@x");
		Location pointYLocation(pointLocation, "@y");
		int x = extract<int>(pointXLocation);
		int y = extract<int>(pointYLocation);
		*/

		int x = extract<int>(pointLocation[XPath("@x")]);
		int y = extract<int>(pointLocation[XPath("@y")]);
		pol.addPoint(Polygon::Point2D(x, y));
	}
}



bool airobots::Polygon::operator ==(const Polygon & other) const {
	try {

		if (points.size() != other.points.size())
			return false;

		for (unsigned int i = 0; i < points.size(); i++) {
			if (points[i].x != other.points[i].x
					|| points[i].y != other.points[i].y) {
				return false;
			}
		}
	} catch (exception & e) {
		return false;
	}
	return true;
}

vector<Polygon::Point2D> Polygon::getPoints() {
	return points;
}



void airobots::Polygon::sanitizeLocation(xmltio::Location & loc) const {
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}

void Polygon::addPoint(Point2D p) {
	points.push_back(p);
}



Polygon::Point2D Polygon::getCentroid() {
	int x = 0;
	int y = 0;
	int count = 0;
	for (unsigned int i = 0; i < points.size(); ++i) {
		Polygon::Point2D &p = points[i];
		x += p.x;
		y += p.y;
		count++;
	}

	Polygon::Point2D point;
	point.x = 0;
	point.y = 0;

	if (count != 0) {
		point.x = x / (double)count;
		point.y = y / (double)count;
	}

	return point;
}



std::string Polygon::getItemTag() {
	return "POINT";
}



std::string Polygon::getBaseTag() {
	return "POLYGON";
}


