#include "PrecisePolygon.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

PrecisePolygon::PrecisePolygon() {
}



PrecisePolygon::PrecisePolygon(const PrecisePolygon &poly) {
	LengthUnit unit = MILLIMETER;
	for (unsigned int i = 0; i < poly.list.size(); ++i) {
		Point2D point = poly.list[i];
		list.push_back(Point2D(point.getX(unit), point.getY(unit), unit));
	}
}



PrecisePolygon::~PrecisePolygon() {
}



std::string PrecisePolygon::getBaseTag() {
	return "PRECISEPOLYGON";
}



std::string PrecisePolygon::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
		+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}



void PrecisePolygon::fillInto(xmltio::Location & loc) const {
	Type::fillInto(loc);

	int n = list.size();
	for (int i = 0; i < n; i++) {
		Location item(Point2D::getTemplate(), Point2D::getBaseTag());
		item = list[i];
		loc.add(item);
	}
}



void PrecisePolygon::fromLocation(const xmltio::Location & loc, PrecisePolygon & pol) {
	Type::fromLocation(loc, pol);

	for (XPathIterator i = XPath(Point2D::getBaseTag()).evaluate(loc); i; ++i) {
		Location pointLocation(*i);
		Point2D p = extract<Point2D>(pointLocation);
		pol.addPoint(p);
	}
}



void PrecisePolygon::sanitizeLocation(xmltio::Location & loc) const {
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}


bool PrecisePolygon::operator ==(const PrecisePolygon & other) const {
	try {
		if (list.size() != other.list.size())
			return false;

		for (unsigned int i = 0; i < list.size(); i++) {
			if (list[i].getX(MILLIMETER) != other.list[i].getX(MILLIMETER)
					|| list[i].getY(MILLIMETER) != other.list[i].getY(MILLIMETER)) {
				return false;
			}
		}
	} catch (exception &e) {
		return false;
	}
	return true;
}



Point2D PrecisePolygon::getCentroid() {
	float x = 0;
	float y = 0;
	int count = 0;
	for (unsigned int i = 0; i < list.size(); ++i) {
		Point2D &p = list[i];
		x += p.getX(MILLIMETER);
		y += p.getY(MILLIMETER);
		count++;
	}

	Point2D point(0, 0, MILLIMETER);

	if (count != 0) {
		point.setX(x / (double)count, MILLIMETER);
		point.setY(y / (double)count, MILLIMETER);
	}

	return point;
}



void PrecisePolygon::addPoint(double x, double y, LengthUnit unit) {
	list.push_back(Point2D(x, y, unit));
}



void PrecisePolygon::addPoint(Point2D point) {
	list.push_back(point);
}

vector<Point2D> PrecisePolygon::getPoints() const {
	return list;
}

