#ifndef RATINGLIST_H_
#define RATINGLIST_H_

#include "Type.h"
#include <map>
#include <string>
#include <math.h>

namespace airobots {

class RatingList: public Type {
public:
	struct XcfHashKey {
		std::string key1, key2;
		bool operator<(const XcfHashKey other) const {
			return key1.compare(key2) < 0;
		}
	};

	RatingList();
	virtual ~RatingList();

	static std::string getBaseTag();
	static std::string getTemplate();

	void sanitizeLocation(xmltio::Location & loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, RatingList &type);

	void addRating(std::string key1, std::string key2, double value);
	void addRating(XcfHashKey key, double value);
	void addRatingWOC(XcfHashKey key, double value);

	double getRating(std::string key1, std::string key2);
	std::map<XcfHashKey, double> getRatings();
	int size();
	void clearValues();
	void setTolerance(double tol);
	void setUpdateNeeded(bool needed);
	void removeID(std::string id);
	bool shouldBeUpdated();

private:
	std::map<XcfHashKey, double> values;
	double tolerance;
	bool updateNeeded;
};

}

#endif /* RATINGLIST_H_ */
