#include "RegionData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string RegionData::getBaseTag() {
	return "REGION";
}

std::string RegionData::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR />" + Timestamp::getTemplate()
			+ "<SCOPE/>" + PrecisePolygon::getTemplate() + "</" + getBaseTag() + ">";
}

RegionData::RegionData() :
	Type() {
}

RegionData::RegionData(std::string generator) :
	Type(generator) {
}

void RegionData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT2D");
	while (XPathIterator i = xp.evaluate(loc[XPath("PRECISEPOLYGON")])) {
		Location l(*i);
		loc.remove(l);
	}
}

void RegionData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);
	Location polyLoc = Location(loc, "PRECISEPOLYGON");
	if (scope == LOCAL) {
		loc[XPath("SCOPE")] = "LOCAL";
	} else {
		loc[XPath("SCOPE")] = "GLOBAL";
	}
	loc[XPath("PRECISEPOLYGON")] = polygon;
}

void RegionData::fromLocation(const xmltio::Location &loc_, RegionData &object) {

	Type::fromLocation(loc_, object);

	string s = extract<string> (loc_[XPath("SCOPE")]);
	if (s == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}

	PrecisePolygon poly = extract<PrecisePolygon>(loc_[XPath("PRECISEPOLYGON")]);
	object.setPolygon(poly);
}

vector<pair<double, double> > RegionData::scalePolygon(vector<pair<double,
		double> > poly, double factor) const {
	for (vector<pair<double, double> >::iterator it = poly.begin(); it
			!= poly.end(); ++it) {
		it->first *= factor;
		it->second *= factor;
	}
	return poly;
}

vector<pair<double, double> > RegionData::getPolygon(LengthUnit unit) const {
	vector<pair<double, double> > poly;
	vector<Point2D> points = polygon.getPoints();
	for (vector<Point2D>::const_iterator i = points.begin(); i != points.end(); ++i) {
		poly.push_back(pair<double, double>(i->getX(unit), i->getY(unit)));
	}

	return poly;

}
void RegionData::setPolygon(const vector<pair<double, double> > &poly,
		LengthUnit unit) {
	vector<pair<double, double> > polyTmp(poly);

	PrecisePolygon newPoly;
	for (vector<pair<double, double> >::iterator i = polyTmp.begin();
			i != polyTmp.end();
			++i) {
		double x = i->first;
		double y = i->second;
		newPoly.addPoint(x, y, unit);
	}

	setPolygon(newPoly);
}

RegionData::Scope RegionData::getScope() const {
	return scope;
}

void RegionData::setScope(const Scope &scope) {
	this->scope = scope;
}

bool RegionData::operator==(const RegionData &other) const {

	return scope == other.scope && polygon == other.getPolygon();
}

PrecisePolygon RegionData::getPolygon() const {
	return polygon;
}

void RegionData::setPolygon(PrecisePolygon poly) {
	polygon = poly;
}
