#include <stdio.h>
#include <sstream>
#include <stdlib.h>

#include "SlamUpdateData.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

std::string SlamUpdateData::getBaseTag() {
	return "SLAMUPDATE";
}

std::string SlamUpdateData::getTemplate() {
	return std::string("<" + getBaseTag() + ">") + "  <GENERATOR />"
			+ Timestamp::getTemplate() + "  <ADDITIONS />"
			+ "  <REMOVALS />" + "</" + getBaseTag() + ">";
}

std::string SlamUpdateData::getGridTemplate() {
	return "<GRID metric_x=\"0\" metric_y=\"0\" />";
}

SlamUpdateData::SlamUpdateData() :
	Type("unknown") {

}

SlamUpdateData::SlamUpdateData(std::string generator, bool init) :
	Type(generator), init(init) {
}

void SlamUpdateData::clear() {
	additions.clear();
	removals.clear();
}

bool SlamUpdateData::isInit() const {
	return init;
}

void SlamUpdateData::setInit(bool init) {
	this->init = init;
}

const SlamUpdateData::list_t & SlamUpdateData::getAdditions() const {
	return additions;
}

const SlamUpdateData::list_t & SlamUpdateData::getRemovals() const {
	return removals;
}

void SlamUpdateData::addAddition(metricCoord_t addition) {
	additions.push_back(addition);
}
void SlamUpdateData::addRemoval(metricCoord_t removal) {
	removals.push_back(removal);
}

void SlamUpdateData::addAdditions(list_t::const_iterator begin,
		list_t::const_iterator end) {
	additions.insert(additions.end(), begin, end);
}

void SlamUpdateData::addRemovals(list_t::const_iterator begin,
		list_t::const_iterator end) {
	removals.insert(removals.end(), begin, end);
}

void SlamUpdateData::sanitizeLocation(xmltio::Location &loc) const {
	Location addLoc = loc[XPath("ADDITIONS")];
	XPath xp("GRID");
	while (XPathIterator i = xp.evaluate(addLoc)) {
		Location l(*i);
		addLoc.remove(l);
	}

	Location remLoc = loc[XPath("REMOVALS")];
	while (XPathIterator i = xp.evaluate(remLoc)) {
		Location l(*i);
		remLoc.remove(l);
	}
}

void SlamUpdateData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	// type
	if (isInit()) {
		loc["type"] = "init";
	} else {
		loc["type"] = "update";
	}

	// add additions
	Location addLoc(loc, "ADDITIONS");
	for (SlamUpdateData::list_t::const_iterator it =
			getAdditions().begin(); it != getAdditions().end(); it++) {
		Location newGrid(SlamUpdateData::getGridTemplate(), "/GRID");
		newGrid["metric_x"] = it->first;
		newGrid["metric_y"] = it->second;
		addLoc.add(newGrid);
	}

	// add removals
	Location remLoc(loc, "REMOVALS");
	for (SlamUpdateData::list_t::const_iterator it = getRemovals().begin(); it
			!= getRemovals().end(); it++) {
		Location newGrid(SlamUpdateData::getGridTemplate(), "/GRID");
		newGrid["metric_x"] = it->first;
		newGrid["metric_y"] = it->second;
		remLoc.add(newGrid);
	}

}

void SlamUpdateData::fromLocation(const xmltio::Location &loc_, SlamUpdateData &sud) {

	Type::fromLocation(loc_, sud);

	string type = extract<string> (loc_["type"]);
	bool init = false;
	if (type == "init") {
		init = true;
	}
	sud.setInit(init);

	for (XPathIterator addIter = XPath("ADDITIONS/GRID").evaluate(loc_); addIter; ++addIter) {
		std::pair<float, float> coordPair;
		coordPair.first = extract<float> ((*addIter)["metric_x"]);
		coordPair.second = extract<float> ((*addIter)["metric_y"]);
		sud.addAddition(coordPair);
	}

	for (XPathIterator remIter = XPath("REMOVALS/GRID").evaluate(loc_); remIter; ++remIter) {
		std::pair<float, float> coordPair;
		coordPair.first = extract<float> ((*remIter)["metric_x"]);
		coordPair.second = extract<float> ((*remIter)["metric_y"]);
		sud.addRemoval(coordPair);
	}

}
