#ifndef SLAMUPDATEDATA_CPP
#define SLAMUPDATEDATA_CPP

#include <stdint.h>
#include <string>
#include <list>
#include <utility>
#include <vector>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "BinarySlamMap.h"

namespace airobots {

/**
 * Representation of binary slam updates.
 */
class SlamUpdateData: public Type {
public:

	static std::string getBaseTag();

	static std::string getTemplate();

	static std::string getGridTemplate();

	SlamUpdateData();
	SlamUpdateData(std::string generator, bool init = false);

	/**
	 * Data type for lists of updates containing metric coordinates.
	 */
	typedef std::pair<float, float> metricCoord_t;
	typedef std::list<metricCoord_t> list_t;

	/**
	 * Clears all additions and removals.
	 */
	void clear();

	bool isInit() const;
	void setInit(bool init);

	const list_t & getAdditions() const;
	const list_t & getRemovals() const;

	void addAddition(metricCoord_t addition);
	void addRemoval(metricCoord_t removal);

	void addAdditions(list_t::const_iterator begin, list_t::const_iterator end);
	void addRemovals(list_t::const_iterator begin, list_t::const_iterator end);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, SlamUpdateData &type);

private:
	list_t additions;
	list_t removals;

	/**
	 * Tells whether this update contains a complete re-initialization.
	 */
	bool init;

};

}

#endif
