 /** ==========================================================================
 * 
 * @file		SplocAngleData.h --
 * @brief		Header file for the angle output of the sploc application.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#ifndef __PERSONDATALIST_H__
#define __PERSONDATALIST_H__

#include <vector>

#include "Type.h"
#include "Ticked.h"

namespace airobots {

/**
 * BTL type for work with the output from the sploc ES speaker localization
 * application.
 */
class SplocAngleData : public Type, public Ticked {

	public:

		/** Constructor. */
		SplocAngleData();

		/** Deconstructor. */
		~SplocAngleData();

		static std::string getBaseTag();
		
		static std::string getTemplate();

		/** Returns all angles to sound percepts. */
		std::vector<float> getAngles();

		/** Sets new angles to sound percepts. */
		void setAngles(std::vector<float> angles);

		void sanitizeLocation(xmltio::Location &loc) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, SplocAngleData &list);

	private:
		/** A list of all computed angles from the sploc application. */
		std::vector<float> angles;

};

}

#endif /** __PERSONDATALIST_H__ */
