/*
 * Timestamp.h
 *
 *  Created on: Jul 22, 2009
 *      Author: jwienke
 */

#ifndef TIMESTAMP_H_
#define TIMESTAMP_H_

#include <string>
#include <sstream>
#include <stdint.h>

#include <xmltio/xmltio.hpp>

namespace airobots {

/**
 * Timestamp associated with data types in milliseconds unix time.
 *
 * @author jwienke
 */
class Timestamp {
public:

	/**
	 * Returns the current system time in unix format as milliseconds.
	 */
	static uint64_t currentTimeMillis();

	inline Timestamp() :
		inserted(currentTimeMillis()), updated(inserted) {
	}

	inline Timestamp(uint64_t inserted) :
		inserted(inserted), updated(inserted) {
	}

	inline Timestamp(uint64_t inserted, uint64_t updated) :
		inserted(inserted), updated(updated) {
	}

	static std::string getBaseTag();

	static std::string getTemplate();

	bool operator==(const Timestamp &other) const;

	uint64_t getInserted() const;
	void setInserted(uint64_t inserted);

	uint64_t getUpdated() const;
	void setUpdated(uint64_t updated);

private:
	/**
	 * Represents the inserting time of the object.
	 */
	uint64_t inserted;

	/**
	 * Represents the last time of update of that object.
	 */
	uint64_t updated;

};

std::ostream& operator<<(std::ostream& o, Timestamp const& timestamp);

}

namespace xmltio {

void convert(const airobots::Timestamp& timestamp, Location& loc);

template<>
airobots::Timestamp extract<airobots::Timestamp>::get() const;

}

#endif /* TIMESTAMP_H_ */
