#include "UnitConverter.h"
#include <math.h>

using namespace airobots;

int UnitConverter::convert(int value, LengthUnit from, LengthUnit to) {
	float asMillimeter = convertFrom((float)value, from);
	return round(convertTo(asMillimeter, to));
}

long UnitConverter::convert(long value, LengthUnit from, LengthUnit to) {
	double asMillimeter = convertFrom((double)value, from);
	return round(convertTo(asMillimeter, to));
}

double UnitConverter::convert(double value, LengthUnit from, LengthUnit to) {
	double asMillimeter = convertFrom(value, from);
	return convertTo(asMillimeter, to);
}

float UnitConverter::convert(float value, LengthUnit from, LengthUnit to) {
	float asMillimeter = convertFrom(value, from);
	return convertTo(asMillimeter, to);
}

double UnitConverter::convert(double value, AngleUnit from, AngleUnit to) {
	double asRadiant = convertFrom(value, from);
	return convertTo(asRadiant, to);
}

float UnitConverter::convert(float value, AngleUnit from, AngleUnit to) {
	float asRadiant = convertFrom(value, from);
	return convertTo(asRadiant, to);
}

long UnitConverter::convert(long value, TimeUnit from, TimeUnit to) {
	long asMicroSeconds = convertFrom(value, from);
	return convertTo(asMicroSeconds, to);
}

double UnitConverter::convertFrom(double value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value * 1000.0;
	case CENTIMETER:
		return value * 10.0;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert from " + unitName(unit));
	}
}

double UnitConverter::convertTo(double value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value / 1000.0;
	case CENTIMETER:
		return value / 10.0;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

float UnitConverter::convertFrom(float value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value * 1000.0f;
	case CENTIMETER:
		return value * 10.0f;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert from " + unitName(unit));
	}
}

float UnitConverter::convertTo(float value, LengthUnit unit) {
	switch (unit) {
	case METER:
		return value / 1000.0f;
	case CENTIMETER:
		return value / 10.0f;
	case MILLIMETER:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

double UnitConverter::convertFrom(double value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / 180.0 * PI;
	case RADIANT:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert from " + unitName(unit));
	}
}

double UnitConverter::convertTo(double value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / PI * 180.0;
	case RADIANT:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

float UnitConverter::convertFrom(float value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / 180.0f * (float) PI;
	case RADIANT:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert from " + unitName(unit));
	}
}

float UnitConverter::convertTo(float value, AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return value / (float) PI * 180.0f;
	case RADIANT:
		return value;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

long UnitConverter::convertTo(long value, TimeUnit unit) {
	switch (unit) {
	case MICROSECONDS:
		return value;
	case MILLISECONDS:
		return value / 1000L;
	case SECONDS:
		return convertTo(value, MILLISECONDS) / 1000L;
	case MINUTES:
		return convertTo(value, SECONDS) / 60L;
	case HOURS:
		return convertTo(value, MINUTES) / 60L;
	case DAYS:
		return convertTo(value, HOURS) / 24L;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

long UnitConverter::convertFrom(long value, TimeUnit unit) {
	switch (unit) {
	case MICROSECONDS:
		return value;
	case MILLISECONDS:
		return value * 1000L;
	case SECONDS:
		return convertFrom(value, MILLISECONDS) * 1000L;
	case MINUTES:
		return convertFrom(value, SECONDS) * 60L;
	case HOURS:
		return convertFrom(value, MINUTES) * 60L;
	case DAYS:
		return convertFrom(value, HOURS) * 24L;
	default:
		throw UnknownUnitException("Don't know how to convert to " + unitName(unit));
	}
}

std::string UnitConverter::unitName(LengthUnit unit) {
	switch (unit) {
	case METER:
		return "METER";
	case CENTIMETER:
		return "CENTIMETER";
	case MILLIMETER:
		return "MILLIMETER";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(AngleUnit unit) {
	switch (unit) {
	case DEGREE:
		return "DEGREE";
	case RADIANT:
		return "RADIANT";
	default:
		return "unknown";
	}
}

std::string UnitConverter::unitName(TimeUnit unit) {
	switch (unit) {
	case MICROSECONDS:
		return "MICROSECONDS";
	case MILLISECONDS:
		return "MILLISECONDS";
	case SECONDS:
		return "SECONDS";
	case MINUTES:
		return "MINUTES";
	case HOURS:
		return "HOURS";
	case DAYS:
		return "DAYS";
	default:
		return "unknown";
	}
}

