/*
 * CameraPositionDataTest.cpp
 *
 *  Created on: Apr 10, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/CameraPositionData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(CameraPositionDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		CameraPositionData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setPan(-0.456);
		original.setTilt(12.45);
		original.setZoom(0.2345);

		Location originalLoc(CameraPositionData::getTemplate(), CameraPositionData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		CameraPositionData parsed = extract<CameraPositionData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getPan(), original.getPan(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTilt(), original.getTilt(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getZoom(), original.getZoom(), TOLERANCE);

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("CameraPosition.xml"), CameraPositionData::getBaseTag());
	CameraPositionData parsed = extract<CameraPositionData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(123.0, parsed.getTilt(), 0.0001);
	BOOST_REQUIRE_CLOSE(0.345, parsed.getPan(), 0.0001);
	BOOST_REQUIRE_CLOSE(0.75, parsed.getZoom(), 0.75);

}

BOOST_AUTO_TEST_SUITE_END()
