/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/CameraTargetData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(CameraTargetDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		CameraTargetData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setAzimuth(-0.456);
		original.setDistance(12.45);
		original.setHeight(1.45);
		original.setZoom(0.2345);

		Location originalLoc(CameraTargetData::getTemplate(), CameraTargetData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		CameraTargetData parsed = extract<CameraTargetData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAzimuth(), original.getAzimuth(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(), original.getDistance(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getHeight(), original.getHeight(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getZoom(), original.getZoom(), TOLERANCE);

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("CameraTarget.xml"), CameraTargetData::getBaseTag());
	CameraTargetData parsed = extract<CameraTargetData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(123.0, parsed.getDistance(), 0.0001);
	BOOST_REQUIRE_CLOSE(2342.0, parsed.getHeight(), 0.0001);
	BOOST_REQUIRE_CLOSE(0.345, parsed.getAzimuth(), 0.0001);
	BOOST_REQUIRE_CLOSE(0.75, parsed.getZoom(), 0.75);

}

BOOST_AUTO_TEST_SUITE_END()
