#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/Face.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(FaceTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Face original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setAngle(21312.78);
		original.setDistance(-1.023);
		original.setHeight(2134.8);
		original.setAngleVariance(0.34);
		original.setDistanceVariance(-12.345);
		original.setHeightVariance(9.72);
		original.setLineOfSight(40);
		original.setFaceWidthAngle(0.55);
		original.setReliability(0.68);
		original.setImagePositionX(452);
		original.setImagePositionY(97);
		original.setImageWidth(455);
		original.setImageHeight(984);


		Location originalLoc(Face::getTemplate(), Face::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		Face parsed = extract<Face>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(), original.getAngle(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(), original.getDistance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getHeight(), original.getHeight(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getAngleVariance(), original.getAngleVariance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistanceVariance(), original.getDistanceVariance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getHeightVariance(), original.getHeightVariance(), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getLineOfSight(), original.getLineOfSight());
		BOOST_REQUIRE_CLOSE(parsed.getFaceWidthAngle(), original.getFaceWidthAngle(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getReliability(), original.getReliability(), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getImagePositionX(), original.getImagePositionX());
		BOOST_REQUIRE_EQUAL(parsed.getImagePositionY(), original.getImagePositionY());
		BOOST_REQUIRE_EQUAL(parsed.getImageWidth(), original.getImageWidth());
		BOOST_REQUIRE_EQUAL(parsed.getImageHeight(), original.getImageHeight());

		BOOST_REQUIRE_EQUAL(original, parsed);
		parsed.setGenerator("trash");
		BOOST_REQUIRE(!(original == parsed));
		parsed.setGenerator(original.getGenerator());
		parsed.setDistance(999.0);
		BOOST_REQUIRE(!(original == parsed));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Face.xml"), Face::getBaseTag());
	Face parsed = extract<Face>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(), tolerance);
	BOOST_REQUIRE_CLOSE((float) -12.34, parsed.getHeight(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getDistance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 0.34, parsed.getAngleVariance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 12.12, parsed.getHeightVariance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) -3.45, parsed.getDistanceVariance(), tolerance);
	BOOST_REQUIRE_EQUAL(30, parsed.getLineOfSight());
	BOOST_REQUIRE_CLOSE((float) 13.56, parsed.getFaceWidthAngle(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 0.67, parsed.getReliability(), tolerance);
	BOOST_REQUIRE_EQUAL((unsigned int) 123, parsed.getImagePositionX());
	BOOST_REQUIRE_EQUAL((unsigned int) 92, parsed.getImagePositionY());
	BOOST_REQUIRE_EQUAL((unsigned int) 200, parsed.getImageWidth());
	BOOST_REQUIRE_EQUAL((unsigned int) 42, parsed.getImageHeight());

}

BOOST_AUTO_TEST_SUITE_END()
