#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#define _USE_MATH_DEFINES
#include <math.h>

#include "btl/LaserInfo.h"
#include "utils.h"

using namespace airobots;
using namespace std;

BOOST_AUTO_TEST_SUITE(LaserInfoTest)

BOOST_AUTO_TEST_CASE(testGetScanNum) {

	BOOST_REQUIRE_EQUAL(0, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, -M_PI / 2.0));
	BOOST_REQUIRE_EQUAL(360, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI / 2.0));
	BOOST_REQUIRE_EQUAL(180, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, 0.0));

	BOOST_REQUIRE_EQUAL(0, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, -M_PI / 2.0 + 0.0001));
	BOOST_REQUIRE_EQUAL(360, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI / 2.0 - 0.0001));

	double nextAngle = -M_PI / 2.0 + (0.5 * M_PI / 180.0);
	BOOST_REQUIRE_EQUAL(1, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, nextAngle));
	BOOST_REQUIRE_EQUAL(1, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, nextAngle - 0.0001));

	double prevAngle = M_PI / 2.0 - (0.5 * M_PI / 180.0);
	BOOST_REQUIRE_EQUAL(359, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, prevAngle));
	BOOST_REQUIRE_EQUAL(359, LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, prevAngle + 0.0001));

	try{
		LaserInfo::calcScanIndex(LaserInfo::SICK_LMS200, M_PI);
		BOOST_FAIL(string("OutOfRangeException has not been thrown although it should have been"));
	} catch (exception e){

	}

}

BOOST_AUTO_TEST_SUITE_END()
