/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include  <iostream>

#include "btl/NavigationGoalData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(NavigationGoalDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		NavigationGoalData original("test", 230.34, -120, 0.623, 3.45, 0.2, NavigationGoalData::GLOBAL);
		original.setTimestamp(Timestamp(123, 456));

		Location originalLoc(NavigationGoalData::getTemplate(), NavigationGoalData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		NavigationGoalData parsed = extract<NavigationGoalData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getType(), original.getType());
		BOOST_REQUIRE_CLOSE(parsed.getDR(), original.getDR(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getDTheta(), original.getDTheta(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTheta(), original.getTheta(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getX(), original.getX(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getY(), original.getY(), TOLERANCE);

	} catch (xmltio::XMLTIO_Exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("NavigationGoal.xml"), NavigationGoalData::getBaseTag());
	NavigationGoalData parsed = extract<NavigationGoalData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 234.5, parsed.getX(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) -6345.42, parsed.getY(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.34, parsed.getTheta(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.77, parsed.getDR(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.5, parsed.getDTheta(), 0.00001);
	BOOST_REQUIRE_EQUAL(NavigationGoalData::GLOBAL, parsed.getType());

}

BOOST_AUTO_TEST_SUITE_END()
