/*
 * PoseDataTest.cpp
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/PoseData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PoseDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		PoseData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setX(222);
		original.setY(333);
		original.setZ(444);
		
		cout << "1" << endl;
		
		Location originalLoc(PoseData::getTemplate(), PoseData::getBaseTag());

		cout << originalLoc.getDocumentText(true) << endl;

		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		PoseData parsed = extract<PoseData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getX(), original.getX());
		BOOST_REQUIRE_EQUAL(parsed.getY(), original.getY());
		BOOST_REQUIRE_EQUAL(parsed.getZ(), original.getZ());

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{
	Location fileContents(readFile("PoseData.xml"), PoseData::getBaseTag());
	PoseData parsed = extract<PoseData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(18, parsed.getX());
	BOOST_REQUIRE_EQUAL(25, parsed.getY());
	BOOST_REQUIRE_EQUAL(34, parsed.getZ());
}

BOOST_AUTO_TEST_SUITE_END()
