#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/RobotState.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(RobotStateTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		RobotState original;
		original.setGenerator("test123321");
		original.setTimestamp(Timestamp(123, 456));
		original.setCurrentState(RobotState::DRIVE_TO);
		original.setPreviousState(RobotState::WAITING);
		original.setTargetName("TestTarget");

		Location originalLoc(RobotState::getTemplate(), RobotState::getBaseTag());
		BOOST_TEST_MESSAGE(originalLoc.getDocumentText(true));
		originalLoc = original;

		RobotState parsed = extract<RobotState>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getCurrentState(), original.getCurrentState());
		BOOST_REQUIRE_EQUAL(parsed.getPreviousState(), original.getPreviousState());
		BOOST_REQUIRE_EQUAL(parsed.getTargetName(), original.getTargetName());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()

