/*
 * SonarDataTest.cpp
 *
 *  Created on: Nov 17, 2010
 *      Author: ltwardon
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/SonarData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(SonarDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		SonarData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setDistanceLeft(1.1);
		original.setDistanceRight(4.2);
		
		Location originalLoc(SonarData::getTemplate(), SonarData::getBaseTag());

		originalLoc = original;

		SonarData parsed = extract<SonarData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getDistanceLeft(), original.getDistanceLeft());
		BOOST_REQUIRE_EQUAL(parsed.getDistanceRight(), original.getDistanceRight());

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{
	Location fileContents(readFile("SonarData.xml"), SonarData::getBaseTag());
	SonarData parsed = extract<SonarData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(1.1, parsed.getDistanceLeft());
	BOOST_REQUIRE_EQUAL(4.2, parsed.getDistanceRight());
}

BOOST_AUTO_TEST_SUITE_END()
