#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/SpeedData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(SpeedDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		SpeedData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setRotation(0.45);
		original.setTranslation(-12.434);
		timeval now;
		gettimeofday(&now, NULL);
		original.setReadTime(now);

		Location originalLoc(SpeedData::getTemplate(), SpeedData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		SpeedData parsed = extract<SpeedData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getRotation(), original.getRotation(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTranslation(), original.getTranslation(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getReadTime(), original.getReadTime());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Speed.xml"), SpeedData::getBaseTag());
	SpeedData parsed = extract<SpeedData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MicroTimestamp(123213412, 335435), parsed.getReadTime());
	BOOST_REQUIRE_CLOSE(-0.34, parsed.getRotation(), 0.0001);
	BOOST_REQUIRE_CLOSE(2.45, parsed.getTranslation(), 0.0001);

}

BOOST_AUTO_TEST_SUITE_END()
