 /** ==========================================================================
 * 
 * @file		SplocAngleDataTest.cpp --
 * @brief		Test suit for sploc angle data test.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * @author dklotz
 * 
 * ========================================================================== 
 **/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>
#include <vector>

#include <sys/time.h>
#include <time.h>


#include "btl/SplocAngleData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(SplocAngleDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	std::vector<float> angles;
	angles.push_back(0.1);
	angles.push_back(0.2);
	angles.push_back(0.3);

	airobots::SplocAngleData original;
	original.setGenerator("Test");
	original.setTimestamp(airobots::Timestamp(1234, 4567));
	original.setAngles(angles);
	timeval now;
	gettimeofday(&now, NULL);
	original.setReadTime(now);

	xmltio::Location loc(airobots::SplocAngleData::getTemplate(),
									airobots::SplocAngleData::getBaseTag());

	loc = original;

	std::cout << loc.getDocumentText(true);

	airobots::SplocAngleData parsed = xmltio::extract<airobots::SplocAngleData>(loc);

	BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
	BOOST_REQUIRE_EQUAL(original.getReadTime().getSeconds(), now.tv_sec);
	BOOST_REQUIRE_EQUAL(original.getReadTime().getMicroSeconds(), now.tv_usec);
	BOOST_REQUIRE_EQUAL(original.getReadTime(), parsed.getReadTime());
	BOOST_REQUIRE_EQUAL(original.getAngles().size(), parsed.getAngles().size());
	BOOST_REQUIRE_CLOSE(original.getAngles()[0], parsed.getAngles()[0], 0.0001);
	BOOST_REQUIRE_CLOSE(original.getAngles()[1], parsed.getAngles()[1], 0.0001);
	BOOST_REQUIRE_CLOSE(original.getAngles()[2], parsed.getAngles()[2], 0.0001);

}


BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	xmltio::Location fileContents(readFile("SplocAngleData.xml"), airobots::SplocAngleData::getBaseTag());
	airobots::SplocAngleData parsed = extract<airobots::SplocAngleData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(1234, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(4567, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MicroTimestamp(123213412, 335435), parsed.getReadTime());
	BOOST_REQUIRE_EQUAL(3, parsed.getAngles().size());
	BOOST_REQUIRE_CLOSE(0.1f, parsed.getAngles()[0], 0.0001);
	BOOST_REQUIRE_CLOSE(0.2f, parsed.getAngles()[1], 0.0001);
	BOOST_REQUIRE_CLOSE(0.3f, parsed.getAngles()[2], 0.0001);

}

BOOST_AUTO_TEST_SUITE_END()
