#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/StateConfiguration.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(StateConfigurationTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		StateConfiguration original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setSystemName("TestName");
		original.addState("test 1234");
		original.addState("Hallo Welt");

		Location originalLoc(StateConfiguration::getTemplate(), StateConfiguration::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		StateConfiguration parsed = extract<StateConfiguration>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getSystemName(), original.getSystemName());
		BOOST_REQUIRE_EQUAL(*(parsed.getStates().begin()), *(original.getStates().begin()));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()
