package de.unibi.airobots.btl.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import org.apache.log4j.Logger;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * Data class for plans from the global path planner component.
 * 
 * @author jwienke
 */
public class GlobalPlan extends Type implements List<NavigationGoalData> {

	private static Logger logger = Logger.getLogger(GlobalPlan.class);

	private List<NavigationGoalData> waypoints = new LinkedList<NavigationGoalData>();

	public GlobalPlan() {
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<NavigationGoalData> iterator() {
		return waypoints.iterator();
	}

	/**
	 * Adds a new waypoint to the plan.
	 * 
	 * @deprecated use {@link #add(NavigationGoalData)} instead.
	 * @param wp
	 *            waypoint to add
	 */
	public void addWaypoint(final NavigationGoalData wp) {
		waypoints.add(wp);
	}

	public static String getBaseTag() {
		return "GLOBALPLAN";
	}

	public static void fromElement(Element element, GlobalPlan type)
			throws ParsingException {

		logger.debug("Parsing plan:\n" + element.toXML());

		Type.fromElement(element, type);

		Nodes nodes = element.query(NavigationGoalData.getBaseTag());
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				NavigationGoalData wp = new NavigationGoalData();
				NavigationGoalData.fromElement(elem, wp);
				type.add(wp);
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		for (PositionData ngd : this) {
			Element ngdElement = new Element(NavigationGoalData.getBaseTag());
			parent.appendChild(ngdElement);
			ngd.fillInto(ngdElement);
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append("Waypoints:\n");
		for (PositionData wp : this) {
			stringBuilder.append("(" + wp.getX() + ", " + wp.getY() + ")\n");
		}

		return stringBuilder.toString();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int size() {
		return waypoints.size();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean add(NavigationGoalData e) {
		return waypoints.add(e);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean addAll(Collection<? extends NavigationGoalData> c) {
		return waypoints.addAll(c);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clear() {
		waypoints.clear();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean contains(Object o) {
		return waypoints.contains(o);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean containsAll(Collection<?> c) {
		return waypoints.containsAll(c);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean isEmpty() {
		return waypoints.isEmpty();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean remove(Object o) {
		return waypoints.remove(o);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean removeAll(Collection<?> c) {
		return waypoints.removeAll(c);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean retainAll(Collection<?> c) {
		return waypoints.retainAll(c);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object[] toArray() {
		return waypoints.toArray();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public <T> T[] toArray(T[] a) {
		return waypoints.toArray(a);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void add(int index, NavigationGoalData element) {
		waypoints.add(index, element);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean addAll(int index, Collection<? extends NavigationGoalData> c) {
		return addAll(index, c);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public NavigationGoalData get(int index) {
		return waypoints.get(index);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int indexOf(Object o) {
		return indexOf(o);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int lastIndexOf(Object o) {
		return lastIndexOf(o);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ListIterator<NavigationGoalData> listIterator() {
		return waypoints.listIterator();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ListIterator<NavigationGoalData> listIterator(int index) {
		return listIterator(index);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public NavigationGoalData remove(int index) {
		return remove(index);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public NavigationGoalData set(int index, NavigationGoalData element) {
		return set(index, element);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<NavigationGoalData> subList(int fromIndex, int toIndex) {
		return subList(fromIndex, toIndex);
	}

}
