package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.common.ElementParser;

public class KatanaPoseData extends Type {

	private double x = 0;
	private double y = 0;
	private double z = 0;
	private double phi = 0;
	private double psi = 0;
	private double theta = 0;

	/**
	 * Constructor.
	 */
	public KatanaPoseData() {
		super();
	}

	public static String getBaseTag() {
		return "POSITION";
	}

	/**
	 * Constructor.
	 */
	public KatanaPoseData(double x, double y, double z, double phi, double psi,
			double theta) {
		super();
		this.x = x;
		this.y = y;
		this.z = z;
		this.psi = psi;
		this.phi = phi;
		this.theta = theta;
	}

	/**
	 * Constructor.
	 */
	public KatanaPoseData(KatanaPoseData kpd) {
		super();
		this.x = kpd.x;
		this.y = kpd.y;
		this.z = kpd.z;
		this.psi = kpd.psi;
		this.phi = kpd.phi;
		this.theta = kpd.theta;
	}

	/**
	 * Returns the x-value of the endeffector position (mm)
	 * 
	 * @return x-value of the endeffector position
	 */
	public double getX() {
		return x;
	}

	/**
	 * Sets the x-value of the endeffector position (mm)
	 * 
	 * @param x
	 *            x-value of the endeffector
	 */
	public void setX(double x) {
		this.x = x;
	}

	/**
	 * Returns the y-value of the endeffector position (mm)
	 * 
	 * @return y-value of the endeffector position
	 */
	public double getY() {
		return y;
	}

	/**
	 * Sets the y-value of the endeffector position (mm)
	 * 
	 * @param y
	 *            y-value of the endeffector
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * Returns the z-value of the endeffector position (mm)
	 * 
	 * @return z-value of the endeffector position
	 */
	public double getZ() {
		return z;
	}

	/**
	 * Sets the z-value of the endeffector position (mm)
	 * 
	 * @param z
	 *            z-value of the endeffector
	 */
	public void setZ(double z) {
		this.z = z;
	}

	/**
	 * Returns the phi-value of the endeffector position (mm)
	 * 
	 * @return phi-value of the endeffector position
	 */
	public double getPhi() {
		return phi;
	}

	/**
	 * Sets the phi-value of the endeffector position (rad)
	 * 
	 * @param phi
	 *            phi-value of the endeffector
	 */
	public void setPhi(double phi) {
		this.phi = phi;
	}

	/**
	 * Returns the psi-value of the endeffector position (mm)
	 * 
	 * @return psi-value of the endeffector position
	 */
	public double getPsi() {
		return psi;
	}

	/**
	 * Sets the psi-value of the endeffector position (rad)
	 * 
	 * @param psi
	 *            psi-value of the endeffector
	 */
	public void setPsi(double psi) {
		this.psi = psi;
	}

	/**
	 * Returns the theta-value of the endeffector position (rad)
	 * 
	 * @return theta-value of the endeffector position
	 */
	public double getTheta() {
		return theta;
	}

	/**
	 * Sets the theta-value of the endeffector position (rad)
	 * 
	 * @param theta
	 *            theta-value of the endeffector
	 */
	public void setTheta(double theta) {
		this.theta = theta;
	}

	public static void fromElement(Element element, KatanaPoseData type) {

		Type.fromElement(element, type);

		Element poseElement = element.getFirstChildElement("P");
		type.setX(ElementParser.getDoubleAttributeValue(poseElement, "x"));
		type.setY(ElementParser.getDoubleAttributeValue(poseElement, "y"));
		type.setZ(ElementParser.getDoubleAttributeValue(poseElement, "z"));

		Element angleElement = element.getFirstChildElement("ANGLE");
		type.setPhi(ElementParser.getDoubleAttributeValue(angleElement, "phi"));
		type.setPsi(ElementParser.getDoubleAttributeValue(angleElement, "psi"));
		type.setTheta(ElementParser.getDoubleAttributeValue(angleElement,
				"theta"));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element poseElement = new Element("P");
		parent.appendChild(poseElement);
		poseElement.addAttribute(new Attribute("x", String.valueOf(getX())));
		poseElement.addAttribute(new Attribute("y", String.valueOf(getY())));
		poseElement.addAttribute(new Attribute("z", String.valueOf(getZ())));

		Element angleElement = new Element("ANGLE");
		parent.appendChild(angleElement);
		angleElement
				.addAttribute(new Attribute("phi", String.valueOf(getPhi())));
		angleElement
				.addAttribute(new Attribute("psi", String.valueOf(getPsi())));
		angleElement.addAttribute(new Attribute("theta", String
				.valueOf(getTheta())));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "KatanaPoseData [x = " + x + ", y = " + y + ", z = " + z
				+ ", phi = " + phi + ", psi = " + psi + ", theta = " + theta
				+ "]";
	}

}
