package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.MicroTimestamp;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * This class is used to store data from the odometry sensors. It holds the
 * xPosition[m], yPosition[m] and yaw[rad] of the robot in world coordinates. It
 * also contains the time of these sensor values as a timestamp.
 * 
 * @author unknown
 * @author jwienke
 */
public class OdometryData extends PositionData implements Ticked {

	private MicroTimestamp readTime = new MicroTimestamp();

	public OdometryData() {
		super();
	}

	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#ODOMETRY# ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("x: " + getX() + "; ");
		strb.append("y: " + getY() + "; ");
		strb.append("yaw: " + getYaw() + ";");
		return strb.toString();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

	}

	public static String getBaseTag() {
		return "ROBOTPOSITION";
	}

	public static void fromElement(Element element, OdometryData type)
			throws ParsingException {
		PositionData.fromElement(element, type);

		Nodes readTimeNodes = element.query(MicroTimestamp.getBaseTag());
		if (readTimeNodes.size() < 1
				|| !(readTimeNodes.get(0) instanceof Element)) {
			throw new ParsingException("Cannot find element with name "
					+ MicroTimestamp.getBaseTag());
		}

		MicroTimestamp.fromElement((Element) readTimeNodes.get(0), type
				.getReadTime());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
