package de.unibi.airobots.btl.data;

import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.PrecisePolygon3D;

public class Path3DData extends Type {

	public static final String SCOPE_TAG_NAME = "SCOPE";

	public enum Scope {
		LOCAL, GLOBAL
	};

	/**
	 * Position of the object in the sensor's image.
	 */
	private PrecisePolygon3D polygon = new PrecisePolygon3D();

	private Scope scope = Scope.GLOBAL;

	public Path3DData() {
	}

	public Path3DData(PrecisePolygon3D polygon) {
		this.polygon = polygon;
	}

	public Path3DData(PrecisePolygon3D polygon, Scope scope) {
		this.polygon = polygon;
		this.scope = scope;
	}

	public Path3DData(Scope scope) {
		this.scope = scope;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PATH3D";
	}

	public PrecisePolygon3D getPolygon() {
		return polygon;
	}

	public void setPolygon(PrecisePolygon3D polygon) {
		this.polygon = polygon;
	}

	public Scope getScope() {
		return scope;
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// polygon
		Element polygonElem = new Element(PrecisePolygon3D.getBaseTag());
		polygon.fillInto(polygonElem);

		Element scopeElem = new Element(SCOPE_TAG_NAME);
		scopeElem.appendChild(scope.name());

		parent.appendChild(polygonElem);
		parent.appendChild(scopeElem);
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with
	 *            all the information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, Path3DData type)
			throws ParsingException {

		Type.fromElement(dataElement, type);

		type.setPolygon(PrecisePolygon3D.fromElement(dataElement
				.getFirstChildElement(PrecisePolygon3D.getBaseTag())));
		type.setScope(Scope.valueOf(dataElement.getFirstChildElement(
				SCOPE_TAG_NAME).getValue()));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp()
				+ " scope: " + scope + " Polygon: " + polygon + "]";

	}
}
