package de.unibi.airobots.btl.data;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

public class PerceptList<T extends Type> extends Type {
	
	private Class<T> perceptType;
	private List<T> percepts = new ArrayList<T>();
	
	private static Logger logger = Logger.getLogger(Type.class);
	
	public PerceptList(Class<T> type) {
		perceptType = type;
	}
	
	public PerceptList(Class<T> type, List<T> percepts) {
		perceptType = type;
		this.percepts = new ArrayList<T>(percepts);
	}
	
	Class<T> getPerceptType() {
		return perceptType;
	}
	
	List<T> getPercepts() {
		return percepts;
	}
	
	void addPercept(T percept) {
		percepts.add(percept);
	}
	
	public static String getBaseTag() {
		return "PERCEPTS";
	}

	public void fillInto(Element parent) {
		super.fillInto(parent);
		
		for (T p : percepts) {
			
			try {
				
				String baseTag = (String) perceptType.getMethod("getBaseTag").invoke(null);
				Element perceptElement = new Element(baseTag);
				p.fillInto(perceptElement);
				parent.appendChild(perceptElement);
				
			} catch (IllegalArgumentException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (SecurityException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (IllegalAccessException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (InvocationTargetException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			} catch (NoSuchMethodException e) {
				logger.fatal("Reflection error.");
				logger.debug(e);
				assert false : "Reflection error.";
			}
			
		}
	}
	
	public static <T extends Type> void fromElement(Element element, PerceptList<T> type) {
		
		Type.fromElement(element, type);
		
		try {
			
			String baseTag = (String) type.getPerceptType().getMethod("getBaseTag").invoke(null);
			Nodes nodes = element.query(baseTag);			
			
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
			
				if (node instanceof Element) {
					Element perceptElement = (Element) node;
					
					T parsed = type.getPerceptType().newInstance();
					Method fromElement = type.getPerceptType().getMethod("fromElement", new Class[] {Element.class, type.getPerceptType()});
					fromElement.invoke(null, perceptElement, parsed);
					
					type.addPercept(parsed);
				}
			}
		
		} catch (InstantiationException e) {
			logger.fatal("Reflection error. Types must have a default constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (IllegalAccessException e) {
			logger.fatal("Reflection error. Types must have a default constructor.");
			logger.debug(e);
			assert false : "Types must have a default constructor.";
		} catch (SecurityException e) {
			logger.fatal("Reflection error. SecurityException");
			logger.debug(e);
			assert false : "Reflection error. SecurityException";
		} catch (NoSuchMethodException e) {
			logger.fatal("Reflection error. NoSuchMethodException");
			logger.debug(e);
			assert false : "Reflection error.";
		} catch (IllegalArgumentException e) {
			logger.fatal("Reflection error. IllegalArgumentException");
			logger.debug(e);
			assert false : "Reflection error. IllegalArgumentException";
		} catch (InvocationTargetException e) {
			logger.fatal("Reflection error. InvocationTargetException");
			logger.debug(e);
			assert false : "Reflection error. InvocationTargetException";
		}
	}
}
