package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.airobots.btl.common.ElementParser;

public class PoseData extends Type {

	private double x = 0;
	private double y = 0;
	private double z = 0;

	/**
	 * Constructor.
	 */
	public PoseData() {
		super();
	}

	public static String getBaseTag() {
		return "currentarmcoord";
	}

	/**
	 * Constructor.
	 */
	public PoseData(double x, double y, double z) {
		super();
		this.x = x;
		this.y = y;
		this.z = z;

	}

	
	/**
	 * Returns the x-value of the endeffector position (mm)
	 * 
	 * @return x-value of the endeffector position
	 */
	public double getX() {
		return x;
	}

	/**
	 * Sets the x-value of the endeffector position (mm)
	 * 
	 * @param x
	 *            x-value of the endeffector
	 */
	public void setX(double x) {
		this.x = x;
	}

	/**
	 * Returns the y-value of the endeffector position (mm)
	 * 
	 * @return y-value of the endeffector position
	 */
	public double getY() {
		return y;
	}

	/**
	 * Sets the y-value of the endeffector position (mm)
	 * 
	 * @param y
	 *            y-value of the endeffector
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * Returns the z-value of the endeffector position (mm)
	 * 
	 * @return z-value of the endeffector position
	 */
	public double getZ() {
		return z;
	}

	/**
	 * Sets the z-value of the endeffector position (mm)
	 * 
	 * @param z
	 *            z-value of the endeffector
	 */
	public void setZ(double z) {
		this.z = z;
	}

	public static void fromElement(Element element, PoseData type) {

		Type.fromElement(element, type);

		Element poseElement = element.getFirstChildElement("currentxyz");
		type.setX(ElementParser.getDoubleAttributeValue(poseElement, "x"));
		type.setY(ElementParser.getDoubleAttributeValue(poseElement, "y"));
		type.setZ(ElementParser.getDoubleAttributeValue(poseElement, "z"));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element poseElement = new Element("currentxyz");
		parent.appendChild(poseElement);
		poseElement.addAttribute(new Attribute("x", String.valueOf(getX())));
		poseElement.addAttribute(new Attribute("y", String.valueOf(getY())));
		poseElement.addAttribute(new Attribute("z", String.valueOf(getZ())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "PoseData [x = " + x + ", y = " + y + ", z = " + z + "]";
	}

}
