package de.unibi.airobots.btl.data;

import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.PrecisePolygon;

public class RegionData extends Type {

	public static final String SCOPE_TAG_NAME = "SCOPE";

	public enum Scope {
		LOCAL, GLOBAL
	};

	/**
	 * Position of the object in the sensor's image.
	 */
	protected PrecisePolygon polygon = new PrecisePolygon();

	protected Scope scope = Scope.GLOBAL;

	public RegionData() {
	}

	public RegionData(PrecisePolygon polygon) {
		this.polygon = polygon;
	}

	public RegionData(PrecisePolygon polygon, Scope scope) {
		this.polygon = polygon;
		this.scope = scope;
	}

	public RegionData(Scope scope) {
		this.scope = scope;
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "REGION";
	}

	public PrecisePolygon getPolygon() {
		return polygon;
	}

	public void setPolygon(PrecisePolygon polygon) {
		this.polygon = polygon;
	}

	public Scope getScope() {
		return scope;
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// polygon
		Element polygonElem = new Element(PrecisePolygon.getBaseTag());
		polygon.fillInto(polygonElem);

		Element scopeElem = new Element(SCOPE_TAG_NAME);
		scopeElem.appendChild(scope.name());

		parent.appendChild(polygonElem);
		parent.appendChild(scopeElem);
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with
	 *            all the information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, RegionData type)
			throws ParsingException {

		Type.fromElement(dataElement, type);

		type.setPolygon(PrecisePolygon.fromElement(dataElement
				.getFirstChildElement(PrecisePolygon.getBaseTag())));
		type.setScope(Scope.valueOf(dataElement.getFirstChildElement(
				SCOPE_TAG_NAME).getValue()));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp()
				+ " scope: " + scope + " Polygon: " + polygon + "]";

	}
}
