package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

public class Shirt extends Type implements Percept{
	
	@Override
	public String getPerceptName() {
		return "shirt";
	}
	
	public static String getBaseTag() {
		return "SHIRT";
	}
	
	public static void fromElement(Element element, Shirt type) {
		
		Type.fromElement(element, type);
		
		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("POSITION")) {
					type.setAngle((float)ElementParser
							.getDoubleAttributeValue(elem, "angle"));
					type.setAngleVariance((float)ElementParser
							.getDoubleAttributeValue(elem, "angleVariance"));
					type.setDistance((float)ElementParser
							.getDoubleAttributeValue(elem, "distance"));
					type.setDistanceVariance((float)ElementParser
							.getDoubleAttributeValue(elem, "distanceVariance"));
				}
			}
		}
		
	}
	
	public void fillInto(Element parent) {
		
		super.fillInto(parent);

		Element posElement = new Element("POSITION");
		parent.appendChild(posElement);
		posElement.addAttribute(new Attribute("" +
				"angle", String
				.valueOf(getAngle())));
		posElement.addAttribute(new Attribute("" +
				"angleVariance", String
				.valueOf(getAngleVariance())));
		posElement.addAttribute(new Attribute("" +
				"distance", String
				.valueOf(getDistance())));
		posElement.addAttribute(new Attribute("" +
				"distanceVariance", String
				.valueOf(getDistanceVariance())));
		
	}
	
	public void setAngle(float angle) {
		this.angle = angle;
	}

	public float getAngle() {
		return angle;
	}

	public void setAngleVariance(float angleVariance) {
		this.angleVariance = angleVariance;
	}

	public float getAngleVariance() {
		return angleVariance;
	}

	public void setDistance(float distance) {
		this.distance = distance;
	}

	public float getDistance() {
		return distance;
	}

	public void setDistanceVariance(float distanceVariance) {
		this.distanceVariance = distanceVariance;
	}

	public float getDistanceVariance() {
		return distanceVariance;
	}
	
	public boolean equals(Shirt other){
		return (this.getAngle() == other.getAngle())
			&& (this.getAngleVariance() == other.getAngleVariance())
			&& (this.getDistance() == other.getDistance())
			&& (this.getDistanceVariance() == other.getDistanceVariance());
	}
	
	/** The angle to the shirt percept. */
	private float angle;

	/** The variance in the angle value. */
	private float angleVariance;

	/** The distance to the shirt percept. */
	private float distance;

	/** The variance in the distance values. */
	private float distanceVariance;
	
}
