package de.unibi.airobots.btl.data;

import nu.xom.Element;
import de.unibi.airobots.btl.data.Type;

/**
 * Xml type for the state visualizer configuration.
 * 
 * @author jwienke
 */
public class StateChangeRequest extends Type {

	private static final String STATE_ELEMENT = "STATE";
	private String newStateName = "unknown";

	/**
	 * Constructor.
	 */
	public StateChangeRequest() {
	}

	/**
	 * Returns the name of the new state to change to.
	 * 
	 * @return staten name
	 */
	public String getNewStateName() {
		return newStateName;
	}

	/**
	 * Sets the name of the state to change to.
	 * 
	 * @param name
	 *            new name
	 */
	public void setNewStateName(String name) {
		this.newStateName = name;
	}

	public static String getBaseTag() {
		return "STATECHANGE";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element stateElement = new Element(STATE_ELEMENT);
		parent.appendChild(stateElement);
		stateElement.appendChild(getNewStateName());

	}

	public static void fromElement(Element element, StateChangeRequest type) {

		Type.fromElement(element, type);

		type.setNewStateName(element.getFirstChildElement(STATE_ELEMENT)
				.getValue());

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[StateChangeRequest: newStateName = " + getNewStateName() + "]";
	}

}
