package de.unibi.airobots.btl.data.annotation;

import java.util.LinkedList;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.Type;

/**
 * 
 * @author ltwardon
 */
public class MapAnnotation extends Type{

	private LinkedList<MapArea> areas;

	public MapAnnotation(){
		
		areas = new LinkedList<MapArea>();
		
	}
	
	public void addArea(MapArea area){
		areas.add(area);
	}
	
	
	public void setAreas(LinkedList<MapArea> areas) {
		this.areas = areas;
	}

	public LinkedList<MapArea> getAreas() {
		return areas;
	}
	
	public LinkedList<MapArea> getAreasByType(MapArea.MapAreaType areaType) {
		LinkedList<MapArea> areas = new LinkedList<MapArea>();
		for(MapArea area : this.areas) {
			if(area.getType().equals(areaType))
				areas.add(area);
		}
		return areas;
	}
	
	public LinkedList<MapArea> getAreasByName(String name) {
		LinkedList<MapArea> areas = new LinkedList<MapArea>();
		for(MapArea area : this.areas) {
			if(area.getArenaName().equals(name))
				areas.add(area);
		}
		return areas;
	}
	
	public static String getBaseTag() {
		return "ANNOTATION";
	}
	
	public static void fromElement(Element element, MapAnnotation type) {

		Type.fromElement(element, type);
		
		// Area
		Nodes nodes_area = element.query("*");
		for (int i = 0; i < nodes_area.size(); i++) {
			Node node_area = nodes_area.get(i);
			if (node_area instanceof Element) {
				Element elem_area = (Element) node_area;
				if (elem_area.getLocalName().equals("AREA")) {
					MapArea.MapAreaType areaType = MapArea.MapAreaType.valueOf(ElementParser.getAttributeValue(elem_area, "type"));
					MapArea area = new MapArea(areaType,
							ElementParser.getAttributeValue(elem_area, "name"),
							new LinkedList<MapRegion>());
					
					// Region
					Nodes nodes_region = elem_area.query("*");
					for (int j = 0; j < nodes_region.size(); j++) {
						Node node_region = nodes_region.get(j);
						if (node_region instanceof Element) {
							Element elem_region = (Element) node_region;
							if (elem_region.getLocalName().equals("REGION")) {
								MapRegion region = new MapRegion("");
								region.setType(ElementParser.getAttributeValue(elem_region, "type"));
								region.setLabel(ElementParser.getAttributeValue(elem_region, "label"));
								region.setGroup(ElementParser.getAttributeValue(elem_region, "group"));
								region.setHeight(ElementParser.getDoubleAttributeValue(elem_region, "high"));
								
								// Point
								Nodes nodes_point = elem_region.query("*");
								for (int k = 0; k < nodes_point.size(); k++) {
									Node node_point = nodes_point.get(k);
									if (node_point instanceof Element) {
										Element elem_point = (Element) node_point;
										// VIEW_POINT
										if (elem_point.getLocalName().equals("VIEW_POINT")) {
											NavigationGoalData viewPoint = new NavigationGoalData(
													type.getGenerator(),
													ElementParser.getDoubleAttributeValue(elem_point, "x"),
													ElementParser.getDoubleAttributeValue(elem_point, "y"),
													ElementParser.getDoubleAttributeValue(elem_point, "yaw"),
													NavigationGoalData.GoalType.GLOBAL);
											region.setViewPoint(viewPoint);
										}
										// COORD
										else if (elem_point.getLocalName().equals("COORD")) {
											MapPoint.Kind kind = MapPoint.Kind.valueOf(ElementParser.getAttributeValue(elem_point, "kind"));
											MapPoint.Unit unit = MapPoint.Unit.valueOf(ElementParser.getAttributeValue(elem_point, "unit"));
											MapPoint point = new MapPoint(
													ElementParser.getDoubleAttributeValue(elem_point, "x"),
													ElementParser.getDoubleAttributeValue(elem_point, "y"),
													kind,
													unit,
													ElementParser.getAttributeValue(elem_point, "ref"));
											region.addPoint(point);
										}
									}
								}
								
								area.addRegion(region);
								
							}
						}
					}
					
					type.areas.add(area);
					
				}
			}
		}
		
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		
		super.fillInto(parent);
		
		// Area
		for(MapArea area : areas){
			Element elem_area = new Element("AREA");
			parent.appendChild(elem_area);
			elem_area.addAttribute(new Attribute("type", area.getType().toString()));
			elem_area.addAttribute(new Attribute("name", area.getArenaName()));
			
			// Region
			for(MapRegion region : area.getRegions()){
				Element elem_region = new Element("REGION");
				elem_area.appendChild(elem_region);
				elem_region.addAttribute(new Attribute("type", region.getType()));
				elem_region.addAttribute(new Attribute("label", region.getLabel()));
				elem_region.addAttribute(new Attribute("group", region.getGroup()));
				elem_region.addAttribute(new Attribute("high", Double.toString(region.getHeight())));
				
				// VIEW_POINT
				Element elem_view_point = new Element("VIEW_POINT");
				elem_region.appendChild(elem_view_point);
				elem_view_point.addAttribute(new Attribute("x", Double.toString(region.getViewPoint().getX())));
				elem_view_point.addAttribute(new Attribute("y", Double.toString(region.getViewPoint().getY())));
				elem_view_point.addAttribute(new Attribute("yaw", Double.toString(region.getViewPoint().getYaw())));
				
				
				// COORD
				for(MapPoint point : region.getPoints()){
					Element elem_point = new Element("COORD");
					elem_region.appendChild(elem_point);
					elem_point.addAttribute(new Attribute("x", Double.toString(point.getX())));
					elem_point.addAttribute(new Attribute("y", Double.toString(point.getY())));
					elem_point.addAttribute(new Attribute("kind", point.getKind().toString()));
					elem_point.addAttribute(new Attribute("unit", point.getUnit().toString()));
					elem_point.addAttribute(new Attribute("ref", point.getRef()));
				}
				
			}
			
		}
		
	}
	
}
