package de.unibi.airobots.btl.data.annotation;

import java.util.LinkedList;
import java.util.List;

/**
 * 
 * @author jhelbach
 */
public class MapArea {

	public static enum MapAreaType {

		NOGO, ROOM, INTEREST, PERSON, DISTURB, OBJECTS, NO_TYP;

	}

	private MapAreaType type;

	private String arenaName;

	private LinkedList<MapRegion> regions;

	public MapArea(MapArea.MapAreaType type) {
		this.arenaName = "";
		this.regions = new LinkedList<MapRegion>();
		this.type = type;
	}

	public MapArea(MapAreaType type, String arenaName,
			LinkedList<MapRegion> regions) {
		this.type = type;
		this.arenaName = arenaName;
		this.regions = regions;
	}

	public void addRegion(MapRegion region) {

		String label = createNewLabel(region);

		region.setLabel(label);
		regions.add(region);
	}

	private boolean checkRegionLabel(String region) {

		for (MapRegion r : regions) {
			if (r.getLabel().equals(region)) {
				return false;
			}
		}
		return true;
	}

	private String createNewLabel(MapRegion region) {

		int i = 0;
		String label = region.getLabel();

		while (!checkRegionLabel(label)) {
			i++;
			label = label.concat(String.valueOf(i));
		}

		return label;
	}

	public MapArea.MapAreaType getType() {
		return type;
	}

	public void setType(MapArea.MapAreaType type) {
		this.type = type;
	}

	public String getArenaName() {
		return arenaName;
	}

	public void setArenaName(String arenaName) {
		this.arenaName = arenaName;
	}

	public LinkedList<MapRegion> getRegions() {
		return regions;
	}

	public void setRegions(LinkedList<MapRegion> regions) {
		this.regions = regions;
	}

	public void removeRegion(String regionLabel) {

		LinkedList<MapRegion> copy = new LinkedList<MapRegion>(regions);

		for (MapRegion r : copy) {

			if (r.getLabel().equals(regionLabel)) {
				regions.remove(r);
			}

		}

	}

	public MapRegion getRegionByLabel(String regionLabel) {
		for (MapRegion r : regions) {

			if (r.getLabel().equals(regionLabel)) {
				return r;
			}

		}
		return null;

	}

	public List<MapRegion> getRegionByGroup(String group) {

		LinkedList<MapRegion> r = new LinkedList<MapRegion>();

		for (MapRegion mr : regions) {
			if (mr.getGroup().equals(group)) {
				r.add(mr);
			}
		}

		return r;

	}

}
