package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;

import java.util.ArrayList;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class AnchorTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		Anchor original = new Anchor();

		AnchorAttribute attr1 = new AnchorAttribute("size");
		attr1.addValue("width", 10.5);
		attr1.addValue("height", 20.5);

		original.addAttribute(attr1);

		Trajectory trajectory = new Trajectory(	);

		AnchorAttribute attr2 = new AnchorAttribute("size");
		attr2.addValue("angle", 10.5);
		attr2.addValue("distance", 20.5);

		AnchorAttribute attr3 = new AnchorAttribute("size");
		attr3.addValue("angle", 13.5);
		attr3.addValue("distance", 23.5);		

		trajectory.addSegment(attr2, attr3, 1234, 4567, 0);

		original.setTrajectory(trajectory);

		Document doc = new Document(new Element(Anchor.getBaseTag()));	
		original.fillInto(doc.getRootElement());

		logger.info(doc.toXML());

		Anchor parsed = new Anchor();
		Anchor.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getAttribute("size"), parsed.getAttribute("size"));

		logger.info("Original attribute: " + original.getAttribute("size").toString());
		logger.info("Parsed attribute: " + parsed.getAttribute("size").toString());

		Trajectory parsedTr = parsed.getTrajectory();
		assertEquals(original.getTrajectory().getSegment(0).getStartPoint(), 
									parsed.getTrajectory().getSegment(0).getStartPoint());
		assertEquals(original.getTrajectory().getSegment(0).getEndPoint(),
									parsed.getTrajectory().getSegment(0).getEndPoint());
		assertEquals(original.getTrajectory().getSegment(0).getStartTime(),
									parsed.getTrajectory().getSegment(0).getStartTime());
		assertEquals(original.getTrajectory().getSegment(0).getEndTime(),
									parsed.getTrajectory().getSegment(0).getEndTime());
		assertEquals(original.getAttribute("size"), parsed.getAttribute("size"));

	}

}
