package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;

import java.util.ArrayList;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class CompositeAnchorTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {
	
		CompositeAnchor original = new CompositeAnchor();
		original.setSymbol("Person");

		AnchorAttribute attr1 = new AnchorAttribute("size");
		attr1.addValue("width", 10d);
		attr1.addValue("height", 20d);

		original.addAttribute(attr1);

		AnchorAttribute attr2 = new AnchorAttribute("position");
		attr2.addValue("angle", 0.1d);
		attr2.addValue("distance", 104.5);

		original.addAttribute(attr2);

		original.addXcfId("34");
		original.addXcfId("2");

		original.addField("legs");
		original.addField("face");

		original.addAssignment("10", 0.5d);
		original.addAssignment("12", 0.4d);

		Document doc = new Document(new Element(CompositeAnchor.getBaseTag()));
		original.fillInto(doc.getRootElement());

		logger.info(doc.toXML());

		CompositeAnchor parsed = new CompositeAnchor();
		CompositeAnchor.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getSymbol(), parsed.getSymbol());
		assertEquals(original.getAttributes().get("size"), parsed.getAttributes().get("size"));
		assertEquals(original.getAttributes().get("position"), parsed.getAttributes().get("position"));
		assertEquals(original.getXcfIds(), parsed.getXcfIds());
		assertEquals(original.getAnchoredFields(), parsed.getAnchoredFields());
	
		assertEquals(original.getRating("10"), parsed.getRating("10"), 0.0001);
		assertEquals(original.getRating("12"), parsed.getRating("12"), 0.0001);


	}

/**
	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("CompositeAnchor.xml"));

		// Create composite anchor that was used to create the xml file.		

		CompositeAnchor original = new CompositeAnchor();
		original.setGenerator("Test");
		
		Anchor legAnchor = new Anchor();
		legAnchor.setType("legs");

		ArrayList<Double> values = new ArrayList<Double>();
		values.add(706.4); values.add(385.9);

		legAnchor.assignValues(values, false);
		legAnchor.getTrajectory().addSegment(values, values, 1234, 2345, 0);

		Anchor faceAnchor = new Anchor();
		faceAnchor.setType("face");

		ArrayList<Double> values2 = new ArrayList<Double>();
		values2.add(0.1); values2.add(1000d);

		faceAnchor.assignValues(values2, false);
		
		original.addAnchor(legAnchor, "legs");
		original.addAnchor(faceAnchor, "face");

		// Compare original anchor with composite anchor parsed from file content.
		CompositeAnchor parsed = new CompositeAnchor();
		CompositeAnchor.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getSymbol(), parsed.getSymbol());
		assertEquals(original.getAnchor("face"), parsed.getAnchor("face"));
		assertEquals(original.getAnchor("legs"), parsed.getAnchor("legs"));
		assertEquals(original.getValues(), parsed.getValues());

	}
*/

}
