package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class DynamicGridMapTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		DynamicGridMap original = new DynamicGridMap();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setHeight(23445);
		original.setOriginX(-345);
		original.setOriginY(123);
		original.setResolution(0.034);
		original.setUri("blubber/bla");
		original.setWidth(34539);

		Document doc = new Document(new Element(DynamicGridMap.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		DynamicGridMap parsed = new DynamicGridMap();
		DynamicGridMap.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getHeight(), parsed.getHeight());
		assertEquals(original.getOriginX(), parsed.getOriginX());
		assertEquals(original.getOriginY(), parsed.getOriginY());
		assertEquals(original.getResolution(), parsed.getResolution(), 0.00001);
		assertEquals(original.getUri(), parsed.getUri());
		assertEquals(original.getWidth(), parsed.getWidth());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("DynamicGridMap.xml"));

		DynamicGridMap parsed = new DynamicGridMap();
		DynamicGridMap.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), DynamicGridMap
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(74, parsed.getOriginX());
		assertEquals(34, parsed.getOriginY());
		assertEquals(0.45, parsed.getResolution(), 0.00001);
		assertEquals(745, parsed.getWidth());
		assertEquals(123, parsed.getHeight());
		assertEquals("test/uri", parsed.getUri());

	}

}
