package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class GoalReachedDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		GoalReachedData original = new GoalReachedData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setStopped(true);

		Document doc = new Document(new Element(GoalReachedData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		GoalReachedData parsed = new GoalReachedData();
		GoalReachedData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.isStopped(), parsed.isStopped());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("GoalReached.xml"));

		GoalReachedData parsed = new GoalReachedData();
		GoalReachedData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), GoalReachedData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(true, parsed.isStopped());

	}

}
