package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author ttoenige
 */
public class KatanaPoseDataTest {
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		KatanaPoseData original = new KatanaPoseData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setX(111);
		original.setY(222);
		original.setZ(333);
		original.setPhi(1.11);
		original.setPsi(0.55);
		original.setTheta(0.77);

		Document doc = new Document(new Element(KatanaPoseData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		KatanaPoseData parsed = new KatanaPoseData();
		KatanaPoseData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getX(), parsed.getX(), 0.00001);
		assertEquals(original.getY(), parsed.getY(), 0.00001);
		assertEquals(original.getZ(), parsed.getZ(), 0.00001);
		assertEquals(original.getPhi(), parsed.getPhi(), 0.00001);
		assertEquals(original.getPsi(), parsed.getPsi(), 0.00001);
		assertEquals(original.getTheta(), parsed.getTheta(), 0.00001);

	}
	
	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("KatanaPoseData.xml"));

		KatanaPoseData parsed = new KatanaPoseData();
		KatanaPoseData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), KatanaPoseData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(232, parsed.getX(), 0.00001);
		assertEquals(343, parsed.getY(), 0.00001);
		assertEquals(454, parsed.getZ(), 0.00001);
		assertEquals(1.23, parsed.getPsi(), 0.00001);
		assertEquals(1.34, parsed.getPhi(), 0.00001);
		assertEquals(0.23, parsed.getTheta(), 0.00001);
		

	}


}
