package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class LegsTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		Legs original = new Legs();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setAngle(0.1);
		original.setAngleVariance(100);
		original.setDistance(200);
		original.setDistanceVariance(250);
		original.setNumInvolved(2);
		original.setInvolved1(1);
		original.setInvolved2(3);
		original.setPair(true);

		Document document = new Document(new Element(Legs.getBaseTag()));
		original.fillInto(document.getRootElement());

		logger.info(document.toXML());

		Legs parsed = new Legs();
		Legs.fromElement(document.getRootElement(), parsed);

		assertEquals(original.getAngle(), parsed.getAngle(), 0.0001);
		assertEquals(original.getAngleVariance(), parsed.getAngleVariance(), 0.0001);
		assertEquals(original.getDistance(), parsed.getDistance(), 0.0001);
		assertEquals(original.getDistanceVariance(), parsed.getDistanceVariance(), 0.0001);
		assertEquals(original.getNumInvolved(), parsed.getNumInvolved(), 0.0001);
		assertEquals(original.getInvolved1(), parsed.getInvolved1(), 0.0001);
		assertEquals(original.getInvolved2(), parsed.getInvolved2(), 0.0001);
		assertEquals(original.isPair(), parsed.isPair());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("Legs.xml"));

		Legs legs = new Legs();
		Legs.fromElement(doc.getRootElement(), legs);

		assertEquals(doc.getRootElement().getLocalName(), Legs.getBaseTag());
		assertEquals("test", legs.getGenerator());
		assertEquals(0.45, legs.getAngle(), 0.0001);
		assertEquals(0.45, legs.getAngleVariance(), 0.0001);
		assertEquals(77.23, legs.getDistance(), 0.0001);
		assertEquals(1.34, legs.getDistanceVariance(), 0.0001);
		assertEquals(2134, legs.getNumInvolved(), 0.0001);
		assertEquals(4543, legs.getInvolved1(), 0.0001);
		assertEquals(1223, legs.getInvolved2(), 0.0001);
		assertEquals(true, legs.isPair());

	}

}
