package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class ObjectDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		
		final long timestamp = System.currentTimeMillis();
		
		ObjectData original = new ObjectData();
		original.setClassLabel("testclass");
		
		ObjectData.Hypothesis hyp = new ObjectData.Hypothesis();
		hyp.setTimestamp(timestamp);
		hyp.setGenerator("test");
		hyp.setReliability(0.1);
		
		original.addHypothesis(hyp);

		Document doc = new Document(new Element(ObjectData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		ObjectData parsed = new ObjectData();
		ObjectData.fromElement(doc.getRootElement(), parsed);
		ObjectData.Hypothesis parsedHyp = parsed.getHypotheses().iterator().next();

		assertEquals(hyp.getGenerator(), parsedHyp.getGenerator());
		assertEquals(hyp.getTimestamp(), parsedHyp.getTimestamp());
		assertEquals(original.getClassLabel(), parsed.getClassLabel());
		assertEquals(hyp.getReliability(), parsedHyp.getReliability(), 0.0001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("ObjectData.xml"));

		ObjectData parsed = new ObjectData();
		ObjectData.fromElement(doc.getRootElement(), parsed);
		ObjectData.Hypothesis parsedHyp = parsed.getHypotheses().iterator().next();
		
		assertEquals(doc.getRootElement().getLocalName(), ObjectData
				.getBaseTag());

		assertEquals("BoostDetector", parsedHyp.getGenerator());
		assertEquals(new Timestamp(253543797l, 253543712l), parsedHyp
				.getTimestamp());
		assertEquals("smacks", parsed.getClassLabel());
		assertEquals(0.6, parsedHyp.getReliability(), 0.002);

	}

}
