package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.PrecisePolygon;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class ObjectPositionDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		

		final long timestamp = System.currentTimeMillis();

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		ObjectPositionData original = new ObjectPositionData();
		original.setClassLabel("testclass");

		ObjectPositionData.Hypothesis originalHyp = new ObjectPositionData.Hypothesis();

		originalHyp.setTimestamp(timestamp);
		originalHyp.setGenerator("test");
		originalHyp.setReliability(0.1);
		originalHyp.setPolygon(poly);
		originalHyp.setReference("world");
		originalHyp.setCoordinateKind("absolute");

		original.addHypothesis(originalHyp);

		Document doc = new Document(new Element(ObjectData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		ObjectPositionData parsed = new ObjectPositionData();
		ObjectPositionData.fromElement(doc.getRootElement(), parsed);
		ObjectPositionData.Hypothesis parsedHyp = parsed
				.getLocationHypotheses().iterator().next();

		assertEquals(originalHyp.getGenerator(), parsedHyp.getGenerator());
		assertEquals(originalHyp.getTimestamp(), parsedHyp.getTimestamp());
		assertEquals(original.getClassLabel(), parsed.getClassLabel());
		assertEquals(originalHyp.getReliability(), parsedHyp.getReliability(),
				0.0001);
		assertEquals(originalHyp.getReference(), parsedHyp.getReference());
		assertEquals(originalHyp.getCoordinateKind(), parsedHyp.getCoordinateKind());
		
		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {
		
		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("ObjectPositionData.xml"));

		ObjectPositionData parsed = new ObjectPositionData();
		ObjectPositionData.fromElement(doc.getRootElement(), parsed);
		ObjectPositionData.Hypothesis parsedHyp = parsed
				.getLocationHypotheses().iterator().next();

		assertEquals(doc.getRootElement().getLocalName(), ObjectPositionData
				.getBaseTag());

		assertEquals("BoostDetector", parsedHyp.getGenerator());
		assertEquals(new Timestamp(253543797l, 253543712l), parsedHyp
				.getTimestamp());
		assertEquals("smacks", parsed.getClassLabel());
		assertEquals(0.6, parsedHyp.getReliability(), 0.002);
		assertEquals("world", parsedHyp.getReference());
		assertEquals("absolute", parsedHyp.getCoordinateKind());
		assertEquals(poly, parsedHyp.getPolygon());
	}

}
