package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.MicroTimestamp;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class OdometryDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		OdometryData original = new OdometryData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setX(-12.34);
		original.setY(34123.45);
		original.setYaw(1.234);

		Document doc = new Document(new Element(OdometryData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		OdometryData parsed = new OdometryData();
		OdometryData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getX(), parsed.getX(), 0.0001);
		assertEquals(original.getY(), parsed.getY(), 0.0001);
		assertEquals(original.getYaw(), parsed.getYaw(), 0.0001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("RobotPosition.xml"));

		OdometryData parsed = new OdometryData();
		OdometryData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), OdometryData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(234.5, parsed.getX(), 0.00001);
		assertEquals(-6345.42, parsed.getY(), 0.00001);
		assertEquals(0.34, parsed.getYaw(), 0.00001);
		assertEquals(new MicroTimestamp(123213412, 335435), parsed
				.getReadTime());

	}

}
