package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.PrecisePolygon;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.RegionData.Scope;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class PlaneDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		PlaneData original = new PlaneData();
		original.setScope(Scope.LOCAL);
		original.setTranslation(1.0, 1.0, 1.0, LengthUnit.MILLIMETER);
		original.setNormalRotationX(2.0, 2.0, 2.0, AngleUnit.RADIANT);

		Document doc = new Document(new Element(PlaneData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PlaneData parsed = new PlaneData();
		PlaneData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getScope(), parsed.getScope());
		assertEquals(original.getPolygon(), parsed.getPolygon());

		assertEquals(original.getTranslationX(LengthUnit.MILLIMETER),
				parsed.getTranslationX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getTranslationY(LengthUnit.MILLIMETER),
				parsed.getTranslationY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getTranslationZ(LengthUnit.MILLIMETER),
				parsed.getTranslationZ(LengthUnit.MILLIMETER), 0.0001);

		assertEquals(original.getNormalRotationX(AngleUnit.RADIANT),
				parsed.getNormalRotationX(AngleUnit.RADIANT), 0.0001);
		assertEquals(original.getNormalRotationY(AngleUnit.RADIANT),
				parsed.getNormalRotationY(AngleUnit.RADIANT), 0.0001);
		assertEquals(original.getNormalRotationZ(AngleUnit.RADIANT),
				parsed.getNormalRotationZ(AngleUnit.RADIANT), 0.0001);

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("PlaneData.xml"));

		PlaneData parsed = new PlaneData();
		PlaneData.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(), PlaneData.getBaseTag());

		assertEquals(new Timestamp(253543797l, 253543712l), parsed.getTimestamp());
		assertEquals(Scope.LOCAL, parsed.getScope());
		assertEquals(poly, parsed.getPolygon());

		assertEquals(1.0, parsed.getTranslationX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(1.0, parsed.getTranslationY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(1.0, parsed.getTranslationZ(LengthUnit.MILLIMETER), 0.0001);

		assertEquals(2.0, parsed.getNormalRotationX(AngleUnit.RADIANT), 0.0001);
		assertEquals(2.0, parsed.getNormalRotationY(AngleUnit.RADIANT), 0.0001);
		assertEquals(2.0, parsed.getNormalRotationZ(AngleUnit.RADIANT), 0.0001);
	}
}
