package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.common.PrecisePolygon;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.RegionData.Scope;
import de.unibi.airobots.btl.units.LengthUnit;

import de.unibi.airobots.btl.TestUtils;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class RegionDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		RegionData original = new RegionData();
		original.setScope(Scope.LOCAL);

		Document doc = new Document(new Element(RegionData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		RegionData parsed = new RegionData();
		RegionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getScope(), parsed.getScope());
		assertEquals(original.getPolygon(), parsed.getPolygon());

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("RegionData.xml"));

		RegionData parsed = new RegionData();
		RegionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(), RegionData
				.getBaseTag());

		assertEquals(new Timestamp(253543797l, 253543712l), parsed
				.getTimestamp());
		assertEquals(Scope.LOCAL, parsed.getScope());
		//assertEquals(poly, parsed.getPolygon());
	}
}
