package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author ltwardon
 */
public class ShirtTest {
	
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		
		try{
		
			Shirt original = new Shirt();
			original.setGenerator("test");
			final long timestamp = System.currentTimeMillis();
			original.setTimestamp(timestamp);
			original.setAngle(0.45f);
			original.setDistance(77.23f);
			original.setAngleVariance(0.345f);
			original.setDistanceVariance(1.34f);
	
			Document doc = new Document(
					new Element(Shirt.getBaseTag()));
			original.fillInto(doc.getRootElement());
			logger.info(doc.toXML());
			
			Shirt parsed = new Shirt();
			Shirt.fromElement(doc.getRootElement(), parsed);
			
			assertEquals(parsed.getGenerator(), original.getGenerator());
			assertEquals(parsed.getTimestamp(), original.getTimestamp());
			assertEquals(parsed.getAngle(), original.getAngle(), 0.0001);
			assertEquals(parsed.getDistance(), original.getDistance(), 0.0001);
			assertEquals(parsed.getAngleVariance(), original.getAngleVariance(), 0.0001);
			assertEquals(parsed.getDistanceVariance(), original.getDistanceVariance(), 0.0001);
		
		} catch(Exception e) {
			fail("Caught exception" + e.getMessage());
		}
			
	}
	
	@Test
	public void fileCompatibility() throws Exception {
		
		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Shirt.xml"));

		Shirt parsed = new Shirt();
		Shirt.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), Shirt
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.45, parsed.getAngle(), 0.0001);
		assertEquals(77.23, parsed.getDistance(), 0.0001);
		assertEquals(0.345, parsed.getAngleVariance(), 0.0001);
		assertEquals(1.34, parsed.getDistanceVariance(), 0.0001);
		
	}

}