package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author ltwardon
 */
public class VoiceTest {
	
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		
		try{
			
			Voice original = new Voice();
			original.setGenerator("test");
			final long timestamp = System.currentTimeMillis();
			original.setTimestamp(timestamp);
			original.setAngle(21312.78f);
			original.setRuntimeDifference(-1.023f);
			original.setAngleVariance(0.45f);
			original.setRuntimeDifferenceVariance(1.34f);
	
			Document doc = new Document(
					new Element(Voice.getBaseTag()));
			original.fillInto(doc.getRootElement());
			logger.info(doc.toXML());
			
			Voice parsed = new Voice();
			Voice.fromElement(doc.getRootElement(), parsed);
			
			assertEquals(parsed.getGenerator(), original.getGenerator());
			assertEquals(parsed.getTimestamp(), original.getTimestamp());
			assertEquals(parsed.getAngle(), original.getAngle(), 0.0001);
			assertEquals(parsed.getRuntimeDifference(), original.getRuntimeDifference(), 0.0001);
			assertEquals(parsed.getAngleVariance(), original.getAngleVariance(), 0.0001);
			assertEquals(parsed.getRuntimeDifferenceVariance(), original.getRuntimeDifferenceVariance(), 0.0001);
			
			assertEquals(parsed, original);
			parsed.setGenerator("trash");
			assertFalse(parsed.equals(original));
			parsed.setGenerator(original.getGenerator());
			parsed.setRuntimeDifference(999.9f);
			assertFalse(parsed.equals(original));
			
		}catch(Exception e){
			fail("Caught exception" + e.getMessage());
		}
	}
	
	@Test
	public void fileCompatibility() throws Exception {
		
		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Voice.xml"));

		Voice parsed = new Voice();
		Voice.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), Voice
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.45, parsed.getAngle(), 0.0001);
		assertEquals(77.23, parsed.getRuntimeDifference(), 0.0001);
		assertEquals(0.345, parsed.getAngleVariance(), 0.0001);
		assertEquals(1.34, parsed.getRuntimeDifferenceVariance(), 0.0001);
		
	}
		
}
